unit ArkanApp;

interface

uses
  Graph,
  CharSet,
  Imager,
  Shell,
  Objects,
  Help,
  Syncron;

const
  HELP_FILE      = 'bbooster.hlp';

  BEGIN_LEVEL    = 0;
  MAX_COLORS     = 7;
  MAX_GLASSES    = 50;
  MAX_PDSIZE     = 6;  {ᨬ ࠧ ࠪ⪨}
  MAX_DIGITS     = 6;
  COLL_FRAMES    = 3;
  GLASS_FRAMES   = 3;
  EXPLODE_FRAMES = 3;
  FRAME_FRAMES   = 6;
  RESTORE_TIME   = 50; {६ ⠭  ௨祩}
  PD_TIME        = 10; {६  -뢠   ࠪ⪨}
  BULLET_TIME    = 2;  { 㫨}
  CHSPEED        = 7;  {  ᪮  -  !}

  MIN_BLSPEED    = 6;  {쭠 ᪮ }
  NORM_BLSPEED   = 8;  {ଠ쭠 ᪮ }
  MAX_BLSPEED    = 19; {ᨬ쭠 ᪮ }

  KEY_ACCEL      = 12; {ࠩ ⠭  筮 ⠭}
  KEY_ACCELTIME  = 4;

  FireDelta : BDelta = (DX : 0; DY : -1; ACCEL : 0; SPEED : 8);

type
  BArkanType      = (T_SINGLE, T_COOPERATIVE);
  BArkanEventType = (EV_NONE, EV_ESCAPE, EV_PDKILLED, EV_LEVELWARP, EV_GAMEOVER);
  BDevice         = record
    Mouse : boolean;
    Keys  : TKeys
  end;

  BStarPtr = ^BStar;
  BStar    = record
    x, y  : integer;
    dy    : integer;
    Color : byte;
    Next  : BStarPtr;
  end;

  BStarsPtr = ^BStars;
  BStars    = object
    private
      Stars : BStarPtr;
    public
      Constructor Init (nStars : integer);
      Procedure   Draw (Ptr : BStarPtr);
      Procedure   Live;
      Destructor  Done;
  end;

  BBarrierPtr = ^BBarrier;
  BBarrier    = object (BObject)
    Constructor Init (bx : integer);
    Procedure   ToDraw (x, y : integer); virtual;
  end;

  BPrizePtr = ^BPrize;
  BPrize    = object (BGravityObject)
    private
      NPrize    : integer;
      PrizeType : BPrizeType;
    public
      Constructor Init (i, j : integer; bdlt : BDelta; PT : BPrizeType);
      Procedure   ToDraw (x, y : integer); virtual;
      Function    GetPrizeType : BPrizeType;
  end;

  BPaddlePtr = ^BPaddle;
  BPaddle    = object (BObject)
    private
      LM, RM             : integer;
      Acc, YAcc, AccTime : integer;
      PaddleSize         : integer;
      OldPaddleSize      : integer;
      LImage, OldLImage  : TImagePtr;
      MImage, OldMImage  : TImagePtr;
      RImage, OldRImage  : TImagePtr;
      Dvc                : BDevice;
      PDMagnity          : boolean;
      PDShooting         : boolean;

      Procedure   PinBall (var PD : BRect);
      Procedure   CatchPrize (var PD : BRect);
      Procedure   PDBoundsHandler (var PD : BRect);
    public
      Constructor Init (bLM, bRM : integer; bDvc : BDevice);
      Procedure   ToLive (var bBounds : BRect); virtual;
      Procedure   ToDraw (x, y : integer); virtual;
      Procedure   PrizeHandler;
      Procedure   SetSize (size : integer);
      Procedure   AccCorrect (var PD : BRect; bx1, bx2 : integer);
      Procedure   SetImage (bLImage, bMImage, bRImage : TImagePtr);
      Function    GetWidth : integer;
      Function    GetSize : integer;
  end;

  BDotPtr = ^BDot;
  BDot    = object (BGravityObject)
    private
      CurColor : integer;
      OldColor : integer;
      Time     : integer;
      nTime    : integer;
    public
      Constructor Init (bx, by : integer; bdlt : BDelta; bTime : integer);
      Procedure   ToLive (var bBounds : BRect); virtual;
      Procedure   ToDraw (x, y : integer); virtual;
  end;

  BCollisionPtr = ^BCollision;
  BCollision    = object (BAnimation)
    Constructor Init (i, j : integer);
    Procedure   DrawFrame (x, y, n : integer); virtual;
  end;

  BRestorePtr = ^BRestore;
  BRestore    = object (BObject)
    private
      Time   : integer;
      ix, iy : integer;
    public
      Constructor Init (i, j : integer);
      Destructor  Done; virtual;
      Procedure   ToLive (var bBounds : BRect); virtual;
  end;

  BGlassPtr = ^BGlass;
  BGlass    = object (BAnimation)
    Constructor Init (i, j : integer);
    Procedure   DrawFrame (x, y, n : integer); virtual;
  end;

  BExplodePtr = ^BExplode;
  BExplode    = object (BAnimation)
    private
      ix, iy   : integer;
      Exploded : boolean;
    public
      Constructor Init (i, j : integer; bexploded : boolean);
      Procedure   ToDie; virtual;
      Procedure   DrawFrame (x, y, n : integer); virtual;
  end;

  BBulletPtr = ^BBullet;
  BBullet    = object (BObject)
    private
      LiveTime : integer;
    public
      Constructor Init (bx, by : integer);
      Procedure   ToLive (var bBounds : BRect); virtual;
      Procedure   ToDraw (x, y : integer); virtual;
  end;

  BBallPtr = ^BBall;
  BBall    = object (BGravityObject)
    private
      Paddle      : BPaddlePtr;
      PaddleX     : integer;
      PaddleTime  : integer;
      ChSpeedTime : integer;
      Image       : TImagePtr;
      OldImage    : TImagePtr;
    public
      Constructor Init (bx, by : integer; bdlt : BDelta; bImage : TImagePtr;
                        bPaddle : BPaddlePtr; PDx : integer);
      Procedure   Paddler (var bBounds : BRect);
      Procedure   CooHandler (bBoundsOld, bBounds : BRect); virtual;
      Procedure   PrizeHandler (bBounds : BRect);
      Procedure   ToDraw (x, y : integer); virtual;
      Procedure   SetPaddle (Ptr : BPaddlePtr);
      Procedure   SetPaddleX (x : integer);
      Procedure   SetImage (bImage : TImagePtr);
      Function    GetPaddle  : BPaddlePtr;
      Function    GetPaddleX : integer;
  end;

  BWallPtr = ^BWall;
  BWall    = object (BField)
    private
      AImages     : array [0..MAX_COLORS] of TImagePtr;
      BImages     : array [0..MAX_COLORS] of TImagePtr;
      CImages     : array [0..MAX_COLORS] of TImagePtr;
      IExplode    : TImagePtr;
      IRestored   : TImagePtr;
      IGolded     : TImagePtr;
      IMoved      : TImagePtr;
    public
      Constructor Init (var GrLib : TGrLib);
      Destructor  Done; virtual;
      Procedure   CollisionHandler (dlt : BDelta; i, j : integer; exploded, moved : boolean); virtual;
      Procedure   DrawBrick (i, j : integer); virtual;
      Procedure   ChoosePrize (i, j : integer; bDlt : BDelta);
      Procedure   WallHandler;
      Procedure   SetOffBricks (BrkType : BBrickType);
      Procedure   ZapBricks;
      Procedure   MultipleExploding;
  end;

  BCounterPtr = ^BCounter;
  BCounter    = object (BObject)
    private
      OldDigits : array [0..MAX_DIGITS] of byte;
      CurScore  : longint;
      CurBonus  : longint;
      Time      : integer;
    public
      Constructor Init (bx, by : integer; Score : longint);
      Procedure   ToLive (var bBounds : BRect); virtual;
      Procedure   ToDraw (x, y : integer); virtual;
      Procedure   AddBonus (bonus : longint);
      Function    GetScore : longint;
  end;

  BArkanApplicationPtr = ^BArkanApplication;
  BArkanApplication    = object
    private
      Lives      : integer;
      Score      : longint;
      Level      : integer;
      Arkan      : BArkanType;
      ArkanEvent : BArkanEventType;
      ArkanDvc1  : BDevice;
      ArkanDvc2  : BDevice;
      GrLib      : TGrLib;

      Procedure   Congratulations;
      Procedure   DrawLives;
      Procedure   DrawFieldOrnament;
      Procedure   WriteMessage (Color : byte; bstring : string; bWait : boolean);
      Procedure   Live;
    public
      Constructor Init (var f : TGrLib; Dvc1, Dvc2 : BDevice;
			bArkan : BArkanType);
      Procedure   Run;
      Function    GetArkanEvent : BArkanEventType;
      Function    GetArkanScore : longint;
      Destructor  Done;
  end;

  procedure CreateDots (dlt : BDelta; i, j, n : integer);

implementation

    var
      ObjList     : BObjectListPtr;
      PaddleList  : BObjectListPtr;
      BallList    : BObjectListPtr;
      PrizeList   : BObjectListPtr;

      PrizeSet    : BPrizeSetPtr;
      Wall        : BWallPtr;
      Counter     : BCounterPtr;
      Barrier     : BBarrierPtr;

      Prizes      : array [0..ord (T_NONE)-1] of TImagePtr;
      CollAnim    : array [0..COLL_FRAMES] of TImagePtr;
      ExplodeAnim : array [0..EXPLODE_FRAMES] of TImagePtr;
      GlassAnim   : array [0..GLASS_FRAMES] of TImagePtr;
      Digits      : array [0..9] of TImagePtr;

      ISBall      : TImagePtr;
      IXBall      : TImagePtr;
      IBall       : TImagePtr;
      ILPaddle    : TImagePtr;
      ILSPaddle   : TImagePtr;
      IMPaddle    : TImagePtr;
      IRPaddle    : TImagePtr;
      IRSPaddle   : TImagePtr;
      IBullet     : TImagePtr;
      ILFrame     : TImagePtr;
      IRFrame     : TImagePtr;
      IGamePic    : TImagePtr;
      ILife       : TImagePtr;
      IBarrier    : TImagePtr;

      MPDWDT      : integer;
      LPDWDT      : integer;
      RPDWDT      : integer;

      GlassTime    : integer;
      GlassFreq    : integer;

      EventCheckFlag : boolean;

      CurShellEvent : integer;
      CurMouseDelta : integer;

constructor BStars.Init (nStars : integer);
var
  Ptr : BStarPtr;
  i   : integer;
begin
  Stars := NIL;
  Randomize;
  SetCharStyle (0, 0, 0);
  for i := 1 to nStars do begin
    New (Ptr);
    with Ptr^ do begin
      Next  := Stars;
      x     := random (80)*8;
      y     := random (GetMaxY);
      dy    := random (4)+1;
      Color := random (8)+8;
      Draw (Ptr)
    end;
    Stars := Ptr
  end
end;

procedure BStars.Draw (Ptr : BStarPtr);
begin
  with Ptr^ do
    WriteString (x, y, color, chr (dy))
end;

procedure BStars.Live;
var
  Ptr : BStarPtr;
begin
  Ptr := Stars;
  while Ptr <> NIL do with Ptr^ do begin
    Draw (Ptr);
    Inc (y, dy);
    if y >= GetMaxY then begin
      y     := 0;
      x     := random (80)*8;
      Color := random (MaxColors)+1
    end;
    Draw (Ptr);
    Ptr := Next
  end
end;

destructor BStars.Done;
var
  Ptr : BStarPtr;
begin
  while Stars <> NIL do begin
    Ptr := Stars^.Next;
    Draw (Ptr);
    Dispose (Stars);
    Stars := Ptr
  end
end;

constructor BBarrier.Init (bx : integer);
begin
  Inherited Init (bx, B_MARGIN-GetImageHgt (IBarrier)-11,
		  bx+GetImageWdt (IBarrier), B_MARGIN-11)
end;

procedure BBarrier.ToDraw (x, y : integer);
begin
  PutImage (x, y, IBarrier, XorPut)
end;

constructor BPrize.Init (i, j : integer; bdlt : BDelta; PT : BPrizeType);
var
  x, y : integer;
  PR   : BRect;
begin
  x         := BrkX (j);
  y         := BrkY (i);
  NPrize    := ord (PT);
  PrizeType := PT;
  Inherited Init (x, y, x + GetImageWdt (Prizes [NPrize]),
                  y + GetImageHgt (Prizes [NPrize]), bdlt);
  SetAccel (16);
  SetSpeed (8)
end;

procedure BPrize.ToDraw (x, y : integer);
begin
  PutImage (x, y, Prizes [NPrize], XorPut);
end;

function BPrize.GetPrizeType : BPrizeType;
begin
  GetPrizeType := PrizeType
end;

constructor BPaddle.Init (bLM, bRM : integer; bDvc : BDevice);
var
  bx : integer;
begin
  Dvc     := bDvc;
  LM      := bLM+1;
  RM      := bRM-1;
  Acc     := 0;
  SetSize (2);
  OldPaddleSize := PaddleSize;
  LImage     := ILPaddle;
  MImage     := IMPaddle;
  RImage     := IRPaddle;
  OldLImage  := LImage;
  OldMImage  := MImage;
  OldRImage  := RImage;
  YAcc       := -10;
  AccTime    := 0;
  PDMagnity  := False;
  PDShooting := False;
  bx         := (LM+RM-GetWidth) div 2;
  Inherited Init (bx, B_MARGIN-GetImageHgt (MImage)-16,
		  bx + GetWidth, B_MARGIN-16);
end;

procedure BPaddle.PDBoundsHandler (var PD : BRect);
begin
  with PD do begin
    if Acc > 0 then
      if (x2 + Acc <= RM) then begin
	Inc (x1, Acc);
	Inc (x2, Acc)
      end else
	AccCorrect (PD, RM, 0);
    if Acc < 0 then
      if (x1 + Acc >= LM) then begin
        Inc (x1, Acc);
	Inc (x2, Acc)
      end else
        AccCorrect (PD, 0, LM)
  end
end;

procedure BPaddle.PinBall (var PD : BRect);
var
  BL     : BRect;
  bDlt   : BDelta;
  i      : integer;
  Ptr    : BListPtr;
  Ball   : BBallPtr;

  procedure ChangeAngle;
  var
    Angle  : integer;
    BallX  : integer;
    PaddX  : integer;
  begin
    BallX := (BL.x1 + BL.x2) div 2;
    PaddX := (PD.x1 + PD.x2) div 2;
    Angle := round ((BallX - PaddX) / GetWidth*130) - 90;
    Ball^.SetAngle (Angle + random (3)-6);
  end;

  procedure CreateDot (Dlt : BDelta; bx, by : integer);
  var
    Dot : BDotPtr;
  begin
    New (Dot, Init (bx, by, dlt, 30));
    with Dot^ do begin
      DeltaMirrorY;
      SetSpeed (8);
      SetAccel (24);
      IncAngle (random (90)-45)
    end;
    ObjList^.AddObject (Dot)
  end;

begin
  BallList^.GetFirstObject (Ptr, BObjectPtr (Ball));
  while Ball <> NIL do begin
    Ball^.GetBounds (BL);
    with Dvc do
      if (KeyCurrent (Keys [FIRE]) and not Mouse) or
	 ((CurShellEvent = cmFire) and Mouse) then
	   if Ball^.GetPaddle = BPaddlePtr (PaddleList^.GetCurObject) then
	     Ball^.SetPaddle (NIL);
    if (BL.x2 > PD.x1) and (BL.x1 < PD.x2) and (BL.y2 > PD.y1) then begin
      if Ball^.GetSpeed = MAX_BLSPEED then begin
          Ball^.GetDelta (bDlt);
          for i := 1 to 3 do
            CreateDot (bDlt, BL.x2, BL.y2)
        end;
      ChangeAngle;
      if PDMagnity then
	Ball^.SetPaddle (BPaddlePtr (PaddleList^.GetCurObject))
    end else if (BL.x2 > PD.x1+Acc) and (BL.x1 < PD.x2+Acc) and (BL.y2 > PD.y1) then begin
      ChangeAngle;
      AccCorrect (PD, BL.x1, BL.x2)
    end;
    BallList^.GetNextObject (Ptr, BObjectPtr (Ball))
  end
end;

procedure BPaddle.CatchPrize (var PD : BRect);
var
  PR     : BRect;
  Ptr    : BListPtr;
  Prize  : BPrizePtr;
begin
  PrizeList^.GetFirstObject (Ptr, BObjectPtr (Prize));
  while Prize <> NIL do with Prize^ do begin
    GetBounds (PR);
    if (PR.x2 > PD.x1) and (PR.x1 < PD.x2) and (PR.y2 > PD.y1) then begin
      ToDie;
      Counter^.AddBonus (ord (GetPrizeType)*5);
      PrizeSet^.AddPrize (GetPrizeType);
      PrizeHandler
    end;
    PrizeList^.GetNextObject (Ptr, BObjectPtr (Prize))
  end
end;

procedure BPaddle.PrizeHandler;
begin
  with PrizeSet^ do begin
    if InAddedSet (T_ALLOFF) then begin
      RemovePrize (T_PDMAGNITY);
      RemovePrize (T_PDSHOOTING);
      RemovePrize (T_PDEXPAND)
    end;
    if InAddedSet (T_PDEXPAND) then
      SetSize (PaddleSize+1);
    if InAddedSet (T_PDSHRINK) then
      SetSize (PaddleSize-1);
    if InRemovedSet (T_PDEXPAND) then
      SetSize (2);
    if InAddedSet (T_PDEXTRASHRINK) then
      SetSize (1);
    if InAddedSet (T_PDSHOOTING) then begin
      SetImage (ILSPaddle, IMPaddle, IRSPaddle);
      PDShooting := True
    end;
    if InRemovedSet (T_PDSHOOTING) then begin
      SetImage (ILPaddle, IMPaddle, IRPaddle);
      PDShooting := False
    end;
    if InAddedSet (T_PDMAGNITY) then
      PDMagnity := True;
    if InRemovedSet (T_PDMAGNITY) then
      PDMagnity := False;
    if InAddedSet (T_XSETOFF) then
      Wall^.SetOffBricks (T_EXPLODED);
    if InAddedSet (T_ZAPBRICKS) then
      Wall^.ZapBricks;
    if InAddedSet (T_MULTEXPLODING) then
      Wall^.MultipleExploding;
    if InAddedSet (T_EXTRABONUS) then with Counter^ do
      AddBonus (1000);
  end
end;

function BPaddle.GetWidth : integer;
begin
  GetWidth := LPDWDT+RPDWDT+MPDWDT*PaddleSize
end;

procedure BPaddle.SetSize (size : integer);
var
  x    : integer;
  half : integer;
begin
  if size in [1..MAX_PDSIZE] then begin
    PaddleSize := size;
    x    := (GetX1 + GetX2) div 2;
    Half := GetWidth div 2;
    if x + Half >= RM then
      x := RM - Half;
    if x - Half <= LM then
      x := LM + Half;
    SetBounds (x - Half, GetY1, x + Half, GetY2);
    Refresh
  end
end;

function BPaddle.GetSize : integer;
begin
  GetSize := PaddleSize
end;

procedure BPaddle.ToLive (var bBounds : BRect);
var
  Bullet : BBulletPtr;
  DeltaX : integer;
begin
  PinBall (bBounds);
  CatchPrize (bBounds);
  Inc (AccTime);
  if AccTime >= KEY_ACCELTIME then begin
    AccTime := 0;
    PDBoundsHandler (bBounds);
    if (BallList^.IsEmptyList) or
       (PrizeSet^.InPrizeSet (T_PDKILL)) then with bBounds do begin
         Inc (y1, YAcc);
         Inc (y2, YAcc);
         Inc (YAcc);
         if y1 >= B_MARGIN then
           ToDie
    end;
    Acc := 0
  end;
  if PDShooting then with Dvc do
      if (KeyPressed (Keys [FIRE]) and not Mouse) or
	 ((CurShellEvent = cmFire) and Mouse) then with bBounds do begin
	   New (Bullet, Init (x1 + 3, y1 - 6));
	   ObjList^.AddObject (Bullet);
	   New (Bullet, Init (x2 - 6, y1 - 6));
	   ObjList^.AddObject (Bullet)
	 end;
  if Dvc.Mouse then
    if CurMouseDelta <> 0 then
      Acc := CurMouseDelta
    else
  else begin
    if KeyCurrent (Dvc.Keys [LEFT]) then
      Acc := -round (KEY_ACCEL * GetMouseAspect);
    if KeyCurrent (Dvc.Keys [RIGHT]) then
      Acc := round (KEY_ACCEL * GetMouseAspect)
  end
end;

procedure BPaddle.ToDraw (x, y : integer);
var
  i : integer;
begin
  PutImage (x, y, OldLImage, XorPut);
  Inc (x, LPDWDT);
  for i := 1 to OldPaddleSize do begin
    PutImage (x, y, OldMImage, XorPut);
    Inc (x, MPDWDT)
  end;
  PutImage (x, y, OldRImage, XorPut);
  OldPaddleSize := PaddleSize;
  OldLImage := LImage;
  OldRImage := RImage;
  OldMImage := MImage
end;

procedure BPaddle.SetImage (bLImage, bMImage, bRImage : TImagePtr);
begin
  LImage := bLImage;
  MImage := bMImage;
  RImage := bRImage;
  Refresh
end;

procedure BPaddle.AccCorrect (var PD : BRect; bx1, bx2 : integer);
begin
  if bx2 > RM then
    bx2 := RM;
  if bx1 < LM then
    bx1 := LM;
  with PD do begin
    if Acc > 0 then begin
      x1 := bx1-x2+x1;
      x2 := bx1
    end;
    if Acc < 0 then begin
      x2 := bx2+x2-x1;
      x1 := bx2
    end;
    Acc := 0
  end
end;

constructor BDot.Init (bx, by : integer; bdlt : BDelta; bTime : integer);
begin
  CurColor := 0;
  OldColor := CurColor;
  Time     := 0;
  nTime    := bTime;
  Inherited Init (bx, by, bx, by, bdlt)
end;

procedure BDot.ToDraw (x, y : integer);
begin
  SetWriteMode (XorPut);
  case OldColor of
    0 : SetColor (White);
    1 : SetColor (Yellow);
    2 : SetColor (LightRed);
    3 : SetColor (Red);
  end;
  OldColor := CurColor;
  Line (x, y, x, y);
  SetWriteMode (NormalPut)
end;

procedure BDot.ToLive (var bBounds : BRect);
begin
  Inc (Time);
  if Time >= nTime then begin
    Time := 0;
    Inc (CurColor);
    if CurColor > 3 then
      ToDie;
  end;
  Inherited ToLive (bBounds)
end;

constructor BCollision.Init (i, j : integer);
begin
  Inherited Init (i, j, COLL_FRAMES, 6)
end;

procedure BCollision.DrawFrame (x, y, n : integer);
begin
  PutImage (x, y, CollAnim [n], XorPut)
end;

constructor BRestore.Init (i, j : integer);
begin
  Time := 0;
  iy   := i;
  ix   := j;
  Inherited Init (0, 0, 0, 0)
end;

destructor BRestore.Done;
var
  Brk : BBrick;
begin
  Brk.BrkType := T_RESTORED;
  Wall^.SetBrick (iy, ix, Brk);
  PutImage (BrkX (ix), BrkY (iy), Wall^.IRestored, XorPut)
end;

procedure BRestore.ToLive (var bBounds : BRect);
begin
  Inc (Time);
  if Time >= RESTORE_TIME*20 then
    ToDie
end;

constructor BGlass.Init (i, j : integer);
begin
  Inherited Init (i, j, GLASS_FRAMES*2+1, 15)
end;

procedure BGlass.DrawFrame (x, y, n : integer);
begin
  if n <= GLASS_FRAMES then
    PutImage (x, y, GlassAnim [n], XorPut)
  else PutImage (x, y, GlassAnim [GLASS_FRAMES*2-n+1], XorPut)
end;

constructor BExplode.Init (i, j : integer; bexploded : boolean);
begin
  iy       := i;
  ix       := j;
  Exploded := bexploded;
  Inherited Init (i, j, EXPLODE_FRAMES, 5)
end;

procedure BExplode.ToDie;
begin
  if Exploded then with Wall^ do begin
    CollisionHandler (FireDelta, iy-1, ix, True, False);
    CollisionHandler (FireDelta, iy, ix+1, True, False);
    CollisionHandler (FireDelta, iy+1, ix, True, False);
    CollisionHandler (FireDelta, iy, ix-1, True, False);
    CollisionHandler (FireDelta, iy-1, ix-1, True, False);
    CollisionHandler (FireDelta, iy-1, ix+1, True, False);
    CollisionHandler (FireDelta, iy+1, ix+1, True, False);
    CollisionHandler (FireDelta, iy+1, ix-1, True, False)
  end;
  Inherited ToDie
end;

procedure BExplode.DrawFrame (x, y, n : integer);
begin
  PutImage (x, y, ExplodeAnim [n], XorPut)
end;

constructor BBullet.Init (bx, by : integer);
begin
  LiveTime := 0;
  Inherited Init (bx, by, bx + GetImageWdt (IBullet),
		  by + GetImageHgt (IBullet))
end;

procedure BBullet.ToLive (var bBounds : BRect);
begin
  Inc (LiveTime);
  if LiveTime >= BULLET_TIME then
    with bBounds do begin
      LiveTime := 0;
      Dec (y1, 4);
      if y1 <= T_MARGIN then
	ToDie;
      if (Wall^.DetectCollision (FireDelta, x1, y1, x2, y2, False)) and
	 not PrizeSet^.InPrizeSet (T_THRUBRICKS) then
	   ToDie
    end
end;

procedure BBullet.ToDraw (x, y : integer);
begin
  PutImage (x, y, IBullet, XorPut)
end;

constructor BBall.Init (bx, by : integer; bdlt : BDelta; bImage : TImagePtr;
                        bPaddle : BPaddlePtr; PDx : integer);
var
  B : BRect;
begin
  Paddle      := bPaddle;
  PaddleX     := PDx;
  PaddleTime  := 0;
  ChSpeedTime := 0;
  Image       := bImage;
  OldImage    := bImage;
  with B do begin
    x1 := bx;
    y1 := by;
    x2 := bx + GetImageWdt (Image);
    y2 := by + GetImageHgt (Image)
  end;
  Paddler (B);
  with B do
    Inherited Init (x1, y1, x2, y2, bdlt)
end;

procedure BBall.Paddler (var bBounds : BRect);
begin
  if Paddle <> NIL then
    with Paddle^ do
      with bBounds do begin
	Inc (PaddleTime);
        if PaddleTime >= PD_TIME*50 then
          SetPaddle (NIL)
        else begin
	  x2 := GetX1+PaddleX+x2-x1;
          y1 := GetY1-y2+y1;
          x1 := GetX1+PaddleX;
          y2 := GetY1;
          BBall.SetBounds (x1, y1, x2, y2)
        end
      end
end;

procedure BBall.CooHandler (bBoundsOld, bBounds : BRect);

  procedure BarrierReflection;
  var
    BR : BRect;
  begin
    if Barrier <> NIL then begin
      Barrier^.GetBounds (BR);
      with BR do begin
        if (bBounds.x1    < X2) and (bBounds.x2    > X1) and
           (bBoundsOld.y1 < Y2) and (bBoundsOld.y2 > Y1) then
             DeltaMirrorX;
        if (bBoundsOld.x1 < X2) and (bBoundsOld.x2 > X1) and
           (bBounds.y1    < Y2) and (bBounds.y2    > Y1) then
             DeltaMirrorY
      end
    end
  end;

  procedure BrickReflection;
  var
    Exploded : boolean;
    Dlt      : BDelta;
    Dot      : BDotPtr;
  begin
    Exploded := PrizeSet^.InPrizeSet (T_BLEXPLODED);
    GetDelta (Dlt);
    if Exploded and (Paddle = NIL) and (ChSpeedTime mod 5 = 0) then
      with bBounds do begin
        New (Dot, Init (x1+random (x2-x1), y1+random (y2-y1), Dlt, 10));
        Dot^.SetAccel (0);
        Dot^.SetSpeed (2);
        ObjList^.AddObject (Dot)
      end;
    with Wall^ do
      if not PrizeSet^.InPrizeSet (T_THRUBRICKS) then begin
        if DetectCollision (Dlt, bBounds.x1, bBoundsOld.y1, bBounds.x2, bBoundsOld.y2,
          Exploded) then DeltaMirrorX;
        if DetectCollision (Dlt, bBoundsOld.x1, bBounds.y1, bBoundsOld.x2, bBounds.y2,
          Exploded) then DeltaMirrorY;
      end else
          DetectCollision (Dlt, bBounds.x1, bBounds.y1,
                           bBounds.x2, bBounds.y2, Exploded)
  end;

begin
  Inherited CooHandler (bBoundsOld, bBounds);
  Paddler (bBounds);
  PrizeHandler (bBounds);
  Inc (ChSpeedTime);
  if ChSpeedTime >= CHSPEED * 130 then begin
    ChSpeedTime := 0;
    if GetSpeed < MAX_BLSPEED then
      SetSpeed (GetSpeed + 1)
  end;
  BarrierReflection;
  BrickReflection
end;

procedure BBall.SetImage (bImage : TImagePtr);
var
  w1, h1 : integer;
  w2, h2 : integer;
  x, y   : integer;
begin
  w1    := GetImageWdt (Image);
  h1    := GetImageHgt (Image);
  w2    := GetImageWdt (bImage);
  h2    := GetImageHgt (bImage);
  Image := bImage;
  x     := GetX1 + (w2 - w1) div 2;
  y     := GetY1 + (h2 - h1) div 2;
  SetBounds (x, y, x + w2, y + h2);
  Refresh
end;

procedure BBall.PrizeHandler (bBounds : BRect);
var
  Ball     : BBallPtr;
  Delta    : BDelta;
begin
  with PrizeSet^ do begin
    if InAddedSet (T_MULTBALL) then begin
      GetDelta (Delta);
      New (Ball, Init (bBounds.x1, bBounds.y1, Delta,
                       Image, GetPaddle, GetPaddleX+5));
      SetPaddleX (PaddleX-5);
      Ball^.IncAngle (5);
      BallList^.AddObject (Ball);
      IncAngle (-5)
    end;
    if InAddedSet (T_SLOWBALL) then
      SetSpeed (MIN_BLSPEED);
    if InAddedSet (T_FASTBALL) then
      SetSpeed (MAX_BLSPEED);
    if InAddedSet (T_ALLOFF) then begin
      RemovePrize (T_GRAVBALL);
      RemovePrize (T_BLEXPLODED);
      RemovePrize (T_THRUBRICKS);
      SetSpeed (NORM_BLSPEED)
    end;
    if InAddedSet (T_BLEXPLODED) then
      SetImage (IXBall);
    if InAddedSet (T_SMALLBALL) then
      RemovePrize (T_BLEXPLODED);
    if InRemovedSet (T_BLEXPLODED) or InRemovedSet (T_SMALLBALL) then
      SetImage (IBall);
    if InAddedSet (T_SMALLBALL) then
      SetImage (ISBall);
    if InAddedSet (T_GRAVBALL) then
      SetAccel (3);
    if InRemovedSet (T_GRAVBALL) then
      SetAccel (0);
    if InRemovedSet (T_PDMAGNITY) or InAddedSet (T_PDEXPAND) or
       InAddedSet (T_PDSHRINK) or InAddedSet (T_PDEXTRASHRINK) then
	 SetPaddle (NIL)
  end
end;

procedure BBall.ToDraw (x, y : integer);
begin
  PutImage (x, y, OldImage, XorPut);
  OldImage := Image
end;

procedure BBall.SetPaddle (Ptr : BPaddlePtr);
var
  PD : BRect;
begin
  Paddle := Ptr;
  if Paddle <> NIL then begin
    Ptr^.GetBounds (PD);
    PaddleTime := 0;
    if GetX1 < PD.x1 then
      SetPaddleX (0)
    else
      if GetX2 > PD.x2 then
        SetPaddleX (Paddle^.GetWidth - GetX2+GetX1)
      else
        SetPaddleX (GetX1-PD.X1)
  end
end;

procedure BBall.SetPaddleX (x : integer);
begin
  PaddleX := x
end;

function BBall.GetPaddle : BPaddlePtr;
begin
  GetPaddle := Paddle
end;

function BBall.GetPaddleX : integer;
begin
  GetPaddleX := PaddleX
end;

constructor BWall.Init (var GrLib : TGrLib);
var
  i : integer;
begin
  Inherited Init;
  for i := 0 to MAX_COLORS do begin
    LoadSprite (GrLib, AImages [i], 'ABRICK' + Str (i));
    LoadSprite (GrLib, BImages [i], 'BBRICK' + Str (i));
    LoadSprite (GrLib, CImages [i], 'CBRICK' + Str (i))
  end;
  LoadSprite (GrLib, IExplode, 'BEXPLODE');
  LoadSprite (GrLib, IRestored, 'BRSTRD');
  LoadSprite (GrLib, IGolded, 'BGOLDED');
  LoadSprite (GrLib, IMoved, 'BMOVED');
end;

destructor BWall.Done;
var
  i : integer;
begin
  Inherited Done;
  FreeImage (IMoved);
  FreeImage (IGolded);
  FreeImage (IRestored);
  FreeImage (IExplode);
  for i := 0 to MAX_COLORS do begin
    FreeImage (BImages [i]);
    FreeImage (AImages [i])
  end
end;

procedure CreateDots (dlt : BDelta; i, j, n : integer);
var
  a : integer;

  procedure AddDot (angle : integer);
  var
    Dot   : BDotPtr;
  begin
    New (Dot, Init (BrkX (j)+BRK_WDT div 2, BrkY (i)+BRK_HGT div 2, Dlt, 40));
    with Dot^ do begin
      IncAngle (angle);
      SetSpeed (8);
      SetAccel (24)
    end;
    ObjList^.AddObject (Dot)
  end;

begin
  for a := 1 to n do
    AddDot ((n-a - n div 2)*15+random (5)-10)
end;

procedure BWall.CollisionHandler (dlt : BDelta; i, j : integer; Exploded, Moved : boolean);
var
  Brk       : BBrick;
  Brk1      : BBrick;
  Collision : BCollisionPtr;
  Explode   : BExplodePtr;
  Restore   : BRestorePtr;
  n, m      : integer;
begin
  with Wall^ do begin
    GetBrick (i, j, Brk);
    n := i; m := j;
    with Brk do if BrkType <> T_EMPTY then begin
      if Exploded and (BrkType <> T_EXPLODED) then begin
        New (Explode, Init (i, j, False));
        ObjList^.AddObject (Explode)
      end;
      case BrkType of
        T_ONCE : begin
                   ChoosePrize (i, j, Dlt);
                   BrkType := T_EMPTY;
                   IncBricksCount (-1);
                   Counter^.AddBonus (5)
                 end;
        T_TWICE,
        T_THREE,
        T_MULTIPLE,
	T_GOLDED   : if Exploded or Moved then begin
	               if BrkType <> T_MULTIPLE then
                         IncBricksCount (-1);
		       Counter^.AddBonus (8);
                       ChoosePrize (i, j, Dlt);
                       BrkType := T_EMPTY
                     end else begin
		       if Color = 0 then
                         CreateDots (Dlt, i, j, 3);
                       if BrkType = T_GOLDED then
                         IncBricksCount (-1)
                       else
                         if PrizeSet^.InPrizeSet (T_THRUBRICKS) then begin
                           Counter^.AddBonus (8);
                           ChoosePrize (i, j, Dlt);
                           if BrkType <> T_MULTIPLE then
                             IncBricksCount (-1);
                           BrkType := T_EMPTY
                         end;
                       if (BrkType <> T_MULTIPLE) and
		          (BrkType <> T_EMPTY) then begin
                            Counter^.AddBonus (10);
                            BrkType := pred (BrkType);
			    ChoosePrize (i, j, Dlt)
			  end;
                       New (Collision, Init (i, j));
                       ObjList^.AddObject (Collision)
                     end;
        T_EXPLODED : begin
                       BrkType := T_EMPTY;
                       IncBricksCount (-1);
                       Counter^.AddBonus (3);
                       New (Explode, Init (i, j, True));
                       ObjList^.AddObject (Explode)
                     end;
        T_RESTORED : begin
                       Counter^.AddBonus (2);
                       BrkType := T_EMPTY;
                       New (Restore, Init (i, j));
                       ObjList^.AddObject (Restore)
                     end;
	T_MOVED    : begin
                       Counter^.AddBonus (2);
		       BrkType := T_EMPTY;
                       New (Collision, Init (i, j));
                         ObjList^.AddObject (Collision);
		       if Exploded then
		         IncBricksCount (-1)
		       else begin
		         n := i + Round (Dlt.dy);
                         m := j + Round (Dlt.dx);
                         GetBrick (n, m, Brk1);
                         CollisionHandler (Dlt, n, m, False, True);
                         if not (n in [0..FIELD_HGT]) or
		            not (m in [0..FIELD_WDT]) or
			    ((Brk1.BrkType <> T_EMPTY) and
			     (Brk1.BrkType <> T_MOVED)) then begin
                              ChoosePrize (i, j, Dlt);
			      n := i; m := j;
                              IncBricksCount (-1);
                              Counter^.AddBonus (20)
                         end;
                         Brk1.BrkType := T_MOVED;
			 SetBrick (n, m, Brk1)
                       end
                     end;
      end;
      DrawBrick (i, j);
      SetBrick (i, j, Brk);
      DrawBrick (n, m)
    end
  end
end;

procedure BWall.ChoosePrize (i, j : integer; bDlt : BDelta);
var
  Prize  : BPrizePtr;
  NPrize : BPrizeType;
  Check  : boolean;
begin
  if PrizeList^.IsEmptyList then
    if random (3) = 0 then begin
      Check  := True;
      NPrize := BPrizeType (0);
      while (NPrize <> T_NONE) and Check do
	if Random (20) <= 15 then
          NPrize := Succ (NPrize)
        else
          Check := False;
      if NPrize <> T_NONE then begin
        CreateDots (bDlt, i, j, 3);
        New (Prize, Init (i, j, bDlt, NPrize));
	PrizeList^.AddObject (Prize)
      end
    end
end;

procedure BWall.DrawBrick (i, j : integer);
var
  Brk  : BBrick;
  x, y : integer;
begin
  x := BrkX (j);
  y := BrkY (i);
  GetBrick (i, j, Brk);
  with Brk do
    case BrkType of
      T_ONCE     : PutImage (x, y, AImages [Color], XorPut);
      T_TWICE    : PutImage (x, y, BImages [Color], XorPut);
      T_THREE,
      T_MULTIPLE : PutImage (x, y, CImages [Color], XorPut);
      T_EXPLODED : PutImage (x, y, IExplode, XorPut);
      T_RESTORED : PutImage (x, y, IRestored, XorPut);
      T_GOLDED   : PutImage (x, y, IGolded, XorPut);
      T_MOVED    : PutImage (x, y, IMoved, XorPut)
    end
end;

procedure BWall.WallHandler;
var
  Brk     : BBrick;
  n, i, j : integer;
  Check   : boolean;
  Time    : integer;
  Ptr     : BGlassPtr;
begin
  Check := False;
  Time  := 0;
  repeat
    i := random (FIELD_HGT+1);
    j := random (FIELD_WDT+1);
    GetBrick (i, j, Brk);
    with Brk do
      if (BrkType = T_TWICE) and (Color = 0) then begin
        New (Ptr, Init (i, j));
        ObjList^.AddObject (ptr);
        Check := True
      end;
    if Time = 100 then Check := True;
    Inc (Time)
  until Check
end;

procedure BWall.SetOffBricks (BrkType : BBrickType);
var
  Brk  : BBrick;
  i, j : integer;
begin
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do begin
      GetBrick (i, j, Brk);
      if Brk.BrkType = BrkType then
        CollisionHandler (FireDelta, i, j, False, False)
    end
end;

procedure BWall.ZapBricks;
var
  Brk  : BBrick;
  i, j : integer;
begin
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do begin
      GetBrick (i, j, Brk);
      with Brk do
        case BrkType of
          T_TWICE,
          T_THREE,
	  T_GOLDED,
	  T_MULTIPLE,
	  T_MOVED : begin
                      if BrkType = T_MULTIPLE then
                        IncBricksCount (1);
	              BrkType := T_ONCE;
                      DrawBrick (i, j);
                      SetBrick (i, j, Brk);
                      DrawBrick (i, j)
		    end;
          T_RESTORED : CollisionHandler (FireDelta, i, j, False, False)
      end
    end
end;

procedure BWall.MultipleExploding;
var
  Brk  : BBrick;
  i, j : integer;

  procedure SetExplodingBrick (i, j : integer);
  var
    Brk : BBrick;
  begin
    if (i in [0..FIELD_HGT]) and (j in [0..FIELD_WDT]) then begin
      GetBrick (i, j, Brk);
      with Brk do
        if BrkType <> T_EXPLODED then begin
          BrkType := T_EXPLODED;
          Color   := 1;
          DrawBrick (i, j);
          SetBrick (i, j, Brk);
          DrawBrick (i, j)
        end
    end
  end;

begin
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do begin
      GetBrick (i, j, Brk);
      if Brk.BrkType = T_EXPLODED then
        Brk.Color := 0;
      SetBrick (i, j, Brk);
    end;
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do begin
      GetBrick (i, j, Brk);
      with Brk do if (Color <> 1) and (BrkType = T_EXPLODED) then begin
        SetExplodingBrick (i-1, j);
        SetExplodingBrick (i+1, j);
        SetExplodingBrick (i, j-1);
        SetExplodingBrick (i, j+1)
      end
    end;
  SetBricksCount (GetNumOfBricks)
end;

constructor BCounter.Init (bx, by : integer; Score : longint);
var
  i : integer;
begin
  for i := 0 to MAX_DIGITS do
    OldDigits [i] := 10;
  CurScore := Score;
  CurBonus := 0;
  Time     := 0;
  Inherited Init (bx, by, 0, 0)
end;

procedure BCounter.AddBonus (bonus : longint);
begin
  CurBonus := CurBonus + bonus
end;

procedure BCounter.ToLive (var bBounds : BRect);
var
  Add : longint;
begin
  if CurBonus <> 0 then begin
    Inc (Time);
    if Time >= 15 then begin
      Time := 0;
      Add  := (CurBonus+15) div 15;
      Dec (CurBonus, Add);
      Inc (CurScore, Add);
      with bBounds do
        ToDraw (x1, y1)
    end
  end
end;

procedure BCounter.ToDraw (x, y : integer);
var
  i : integer;
  n : longint;
  k : integer;
begin
  n := CurScore;
  for i := MAX_DIGITS downto 0 do begin
    k := n mod 10;
    n := n div 10;
    if OldDigits [i] <> k then begin
      if OldDigits [i] <> 10 then
        PutImage (x+10*i, y, Digits [OldDigits [i]], XorPut);
      PutImage (x+10*i, y, Digits [k], XorPut);
      OldDigits [i] := k
    end
  end
end;

function BCounter.GetScore : longint;
begin
  GetScore := CurScore + CurBonus
end;

constructor BArkanApplication.Init (var f : TGrLib; Dvc1, Dvc2 : BDevice;
				    bArkan : BArkanType);
var
  i : integer;
begin
  InitArea (0);
  LoadSprite (f, IBall, 'BALL');
  LoadSprite (f, IXBall, 'XBALL');
  LoadSprite (f, ISBall, 'SBALL');
  LoadSprite (f, ILPaddle, 'LPADDLE');
  LoadSprite (f, ILSPaddle, 'LSPADDLE');
  LoadSprite (f, IMPaddle, 'MPADDLE');
  LoadSprite (f, IRPaddle, 'RPADDLE');
  LoadSprite (f, IRSPaddle, 'RSPADDLE');
  LoadSprite (f, IBullet, 'BULLET');
  LoadSprite (f, IRFrame, 'RFRAME');
  LoadSprite (f, ILFrame, 'LFRAME');
  LoadSprite (f, IGamePic, 'GAMEPIC');
  LoadSprite (f, ILife, 'LIFE');
  LoadSprite (f, IBarrier, 'BARRIER');
  LPDWDT := GetImageWdt (ILPaddle)+1;
  MPDWDT := GetImageWdt (IMPaddle)+1;
  RPDWDT := GetImageWdt (IRPaddle)+1;
  for i := 0 to ord (T_NONE)-1 do
    LoadSprite (f, Prizes [i], 'PRIZE' + Str (i));
  for i := 0 to COLL_FRAMES do
    LoadSprite (f, CollAnim [i], 'BCOLL' + Str (i));
  for i := 0 to EXPLODE_FRAMES do
    LoadSprite (f, ExplodeAnim [i], 'EXPLODE' + Str (i));
  for i := 0 to GLASS_FRAMES do
    LoadSprite (f, GlassAnim [i], 'GLASS' + Str (i));
  for i := 0 to 9 do
    LoadSprite (f, Digits [i], 'DIGIT' + Str (i));
  New (Wall, Init (f));
  Randomize;
  Arkan     := bArkan;
  GlassFreq := 100;
  ArkanDvc1 := Dvc1;
  ArkanDvc2 := Dvc2;
  Score     := 0;
  Level     := BEGIN_LEVEL;
  GrLib     := f;
  Lives     := 2;
  Wall^.SeekField (Level);
  Wall^.LoadNextField
end;

procedure BArkanApplication.DrawLives;
var
  i : integer;
begin
  for i := 1 to Lives do
    PutImage (i*23, 340, ILife, XorPut)
end;

procedure BArkanApplication.DrawFieldOrnament;
var
  i    : integer;
  Name : string;
begin
  for i := 0 to 21 do begin
    PutImage (0, i*16, ILFrame, XorPut);
    PutImage (620, i*16, IRFrame, XorPut)
  end;
  PutImage (560, 0, IGamePic, XorPut);
  DrawLives;
  Name := Wall^.GetFieldName;
  SetCharStyle (0, 0, 0);
  WriteString (620-GetStringWidth (Name), 339, LightGreen, Name)
end;

procedure BArkanApplication.WriteMessage (Color : byte; bstring : string; bWait : boolean);
var
  SavePic : TImagePtr;
  Time    : integer;
  MiddleX : integer;
  i       : integer;
begin
  CreateImage (SavePic, 189, 45);
  GetImage (225, 187, SavePic);
  SetFillStyle (SolidFill, DarkGray);
  Bar (225, 187, 414, 232);
  SetColor (LightGray);
  Rectangle (225, 187, 414, 232);
  SetCharStyle (2, 1, 1);
  MiddleX := (GetMaxX - GetStringWidth (bstring)) div 2+2;
  for i := 1 to Length (bstring) do begin
    WriteString (MiddleX+(i-1)*16, 204, Color, bstring [i]);
    Delay (70)
  end;
  if bWait then begin
    Time := 0;
    repeat
      Inc (Time);
      Delay (100)
    until Time >= 25
  end else
    repeat until AnyKey or (GetShellEvent = cmFire);
  ClearShellKeys;
  ClearShellEvent;
  ClearMouseDelta;
  PutImage (225, 187, SavePic, NormalPut);
  FreeImage (SavePic)
end;

procedure GlassHandler;
begin
  Inc (GlassTime);
  if GlassTime = GlassFreq then begin
    Wall^.WallHandler;
    GlassTime := 0;
    GlassFreq := random (100)+100
  end
end;

procedure LiveAll; far;
begin
  CurMouseDelta := GetMouseDeltaX;
  ClearMouseDelta;
  CurShellEvent := GetShellEvent;
  ClearShellEvent;
  PaddleList^.LiveAll;
  PrizeList^.LiveAll;
  BallList^.LiveAll;
  ObjList^.LiveAll;
  GlassHandler;
  EventCheckFlag := True
end;

procedure DrawAll; far;
begin
  PaddleList^.DrawAll;
  PrizeList^.DrawAll;
  BallList^.DrawAll;
  ObjList^.DrawAll
end;

procedure BArkanApplication.Live;
var
  Paddle    : BPaddlePtr;
  Ball      : BBallPtr;
  WaitEvent : BArkanEventType;
  Time      : integer;

  procedure Born;
  begin
    ClearDevice;
    New (PrizeSet, Init);
    New (PaddleList, Init);
    New (BallList, Init);
    New (PrizeList, Init);
    New (ObjList, Init);
    DrawFieldOrnament;
    Wall^.DrawField;
    New (Counter, Init (20, 0, Score));
    ObjList^.AddObject (Counter);
    ArkanEvent     := EV_NONE;
    WaitEvent      := EV_NONE;
    EventCheckFlag := False;
    Time           := 0;
    if Arkan = T_COOPERATIVE then begin
      New (Barrier, Init ((L_MARGIN+R_MARGIN) div 2));
      New (Paddle, Init (Barrier^.GetX2, R_MARGIN, ArkanDvc2));
      PaddleList^.AddObject (Paddle);
      New (Paddle, Init (L_MARGIN, Barrier^.GetX1, ArkanDvc1))
    end else
      New (Paddle, Init (L_MARGIN, R_MARGIN, ArkanDvc1));
    PaddleList^.AddObject (Paddle);
    with BallList^ do begin
      New (Ball, Init (0, 0, FireDelta, IBall, Paddle, 30));
      Ball^.SetAngle (-110+random (40));
      Ball^.SetAccel (0);
      Ball^.SetSpeed (NORM_BLSPEED);
      AddObject (Ball)
    end;
    WriteMessage (LightMagenta, 'LEVEL '+str (Level+1), True)
  end;

  procedure EventCreater;
  var
    HelpFlag : boolean;
  begin
    if PrizeSet^.InAddedSet (T_WARPLEVEL) then
      ArkanEvent := EV_LEVELWARP;
    if PrizeSet^.InAddedSet (T_EXTRALIFE) then begin
      DrawLives;
      Inc (Lives);
      DrawLives
    end;
    PrizeSet^.ClearSet;
    if WaitEvent <> EV_NONE then begin
      Inc (Time);
      if Time >= 180 then
	ArkanEvent := WaitEvent
    end;
    if KeyPressed (Escape) then
      ArkanEvent := EV_ESCAPE;
    HelpFlag := False;
    repeat
      if KeyPressed (BackSpace) or HelpFlag then
	WriteMessage (White, 'PAUSED', False);
      HelpFlag := KeyCurrent (F1);
      if HelpFlag then
	ToHelp (GrLib, HELP_FILE)
    until not HelpFlag;
    if PaddleList^.IsEmptyList and (WaitEvent = EV_NONE) then
      WaitEvent := EV_PDKILLED;
    if (Wall^.GetBricksCount = 0) and (WaitEvent = EV_NONE) then
      ArkanEvent := EV_LEVELWARP
  end;

  procedure LifeDone;
  begin
    Score := Counter^.GetScore;
    if Arkan <> T_SINGLE then
      Dispose (Barrier, Done);
    Dispose (ObjList, Done);
    Dispose (PrizeList, Done);
    Dispose (BallList, Done);
    Dispose (PaddleList, Done);
    Dispose (PrizeSet)
  end;

begin
  Born;
  repeat
    Syncro (LiveAll, DrawAll);
    if EventCheckFlag then
      EventCreater
  until ArkanEvent <> EV_NONE;
  LifeDone
end;

procedure BArkanApplication.Run;
begin
  repeat
    Live;
    if ArkanEvent = EV_PDKILLED then begin
      Dec (Lives);
      ArkanEvent := EV_NONE;
      if Wall^.GetBricksCount = 0 then
	ArkanEvent := EV_LEVELWARP;
      if Lives < 0 then begin
	ArkanEvent := EV_GAMEOVER;
        WriteMessage (LightRed, 'GAME OVER', True)
      end
    end;
    if ArkanEvent = EV_LEVELWARP then begin
      WriteMessage (Yellow, 'COMPLETED', True);
      Inc (Level);
      Inc (Score, 200);
      if Wall^.EOFields then begin
	Congratulations;
	Level := 0;
        Wall^.SeekField (Level)
      end;
      ArkanEvent := EV_NONE;
      Wall^.LoadNextField
    end
  until ArkanEvent <> EV_NONE
end;

function BArkanApplication.GetArkanEvent : BArkanEventType;
begin
  GetArkanEvent := ArkanEvent
end;

function BArkanApplication.GetArkanScore : longint;
begin
  GetArkanScore := Score
end;

destructor BArkanApplication.Done;
var
  i : integer;
begin
  Dispose (Wall, Done);
  for i := 0 to 9 do
    FreeImage (Digits [i]);
  for i := 0 to GLASS_FRAMES do
    FreeImage (GlassAnim [i]);
  for i := 0 to EXPLODE_FRAMES do
    FreeImage (ExplodeAnim [i]);
  for i := 0 to COLL_FRAMES do
    FreeImage (CollAnim [i]);
  for i := 0 to ord (T_NONE)-1 do
    FreeImage (Prizes [i]);
  FreeImage (IBarrier);
  FreeImage (ILife);
  FreeImage (IGamePic);
  FreeImage (ILFrame);
  FreeImage (IRFrame);
  FreeImage (IBullet);
  FreeImage (IRSPaddle);
  FreeImage (IRPaddle);
  FreeImage (IMPaddle);
  FreeImage (ILSPaddle);
  FreeImage (ILPaddle);
  FreeImage (ISBall);
  FreeImage (IXBall);
  FreeImage (IBall)
end;

procedure BArkanApplication.Congratulations;
var
  i     : integer;
  Stars : BStarsPtr;
begin
  ClearDevice;
  SetCharStyle (5, 3, 2);
  WriteString (200, 50, White, 'END !');
  SetCharStyle (0, 0, 0);
  WriteString (32, 150, Yellow,     '   ࠤ ? " 祬 ", - 㬠 . " ⮬ ", - 㬠 .');
  WriteString (0, 160, LightGreen, '  ᫨ 쥧,    ⮫쪮  -  諨 ,  ⮬-  ');
  WriteString (0, 170, LightRed,   '  ࠤ. ...  "-      墠⠥", -  㬠 .');
  WriteString (0, 180, LightMagenta, ' 㤥 ࠢ, ⠪    (易 ?)  ࠢ  ⠪  :');
  WriteString (40, 200, LightBlue,  '⠪, ---------   ⨬ ⥫ ᮡ⨥,');
  WriteString (40, 210, LightCyan,  '   ⠪      ,  ,');
  WriteString (40, 220, White,      'ᮢ  ஢   ...     ');
  WriteString (40, 230, Yellow,     ' ᭥ :-(');
  WriteString (0, 250, LightGreen, ', ᫨  ⫨,     ⥪,  ⠩');
  WriteString (0, 260, LightRed,   '  ᠬ 砫 :-)');
  WriteString (40, 280, LightMagenta, 'P.S.   ⫨ ?  ⮣ ࠩ ᭠砫 !');
  WriteString (208, 330, White,     '   ');
  New (Stars, Init (20));
  repeat
    Stars^.Live;
    Delay (20)
  until AnyKey or (GetShellEvent = cmFire);
  Dispose (Stars, Done)
end;

begin
end.
