Program BallBooster;

uses
  Crt,
  Graph,
  Imager,
  Shell,
  ArkanApp,
  CharSet,
  Help,
  Syncron;

{$I-}

const
  CFG_FILE     = 'bbooster.cfg';

  BBOOSTER_CPS = 100;

  MAX_SCORES   = 10;
  NAME_LENGTH  = 40;

  Names     : array [1..10] of string [NAME_LENGTH] =
    ('Nine One One', 'Adventurous', 'Virtual Humanity',
     'Neutral Zone', 'Inferiority Complex', 'Late Shift', 'Foreign Skys',
     'Before The Ways Cross', 'Northern Sky', 'Route Nova');

  ColorData : array [0..5] of byte = (LightRed, Yellow, LightGreen,
				      LightCyan, LightBlue, LightMagenta);

  cmStartGame   = 1;
  cmOptions     = 2;
  cmHallOfFame  = 3;
  cmHelp        = 4;
  cmQuit        = 5;

  cmMSensDec    = 6;
  cmMSensInc    = 7;
  cmSingle      = 8;
  cmCooperative = 9;
  cmDevice1     = 10;
  cmDevice2     = 11;
  cmRedefine1   = 12;
  cmRedefine2   = 13;

type
  BScore = record
    Name  : string [NAME_LENGTH];
    Score : longint;
  end;

  BScoreTable = array [1..MAX_SCORES] of BScore;

  BConfig = record
    Tables    : array [BArkanType] of BScoreTable;
    MAspect   : Real;
    ArkanType : BArkanType;
    Device1   : BDevice;
    Device2   : BDevice;
  end;

  BConfigFile = record
    CFG : BConfig;
    CRC : longint;
  end;

  BBoosterApplicationPtr = ^BBoosterApplication;
  BBoosterApplication    = object
    private
      Config    : BConfig;
      GrLib     : TGrLib;

      Procedure Options;
      Procedure QuitConfirm;
      Procedure ShowMenu;
      Procedure HallOfFame (NewSc : longint);
      Procedure ReadConfig;
      Procedure WriteConfig;
      Procedure CreateConfig;
      Function  CalcCRC (Cfg : BConfig) : longint;
      Function  GetPlace (Sc : longint) : integer;
    public
      Constructor Init;
      Destructor  Done;
      Procedure   Run;
  end;

var
  BoosterApplication : BBoosterApplicationPtr;

function BBoosterApplication.CalcCRC (Cfg : BConfig) : longint;
var
  CRC : longint;
  Ptr : ^byte;
  i   : integer;
begin
  Ptr := @Cfg;
  CRC := 0;
  for i := 1 to SizeOf (Config) do begin
    Inc (CRC, Ptr^);
    Inc (Ptr)
  end;
  CRC := CRC xor $89ABCDEF + 54321;
  CalcCRC := CRC
end;

procedure BBoosterApplication.CreateConfig;
var
  ConfigFile : BConfigFile;
  f          : file of BConfigFile;
  i          : BArkanType;
  j          : integer;
begin
  Assign (f, CFG_FILE);
  Rewrite (f);
  with Config do begin
    for i := T_SINGLE to T_COOPERATIVE do
      for j := 1 to MAX_SCORES do with Tables [i][j] do begin
	Name  := Names [j];
	Score := (MAX_SCORES - j+1)*1000
      end;
    with Device1 do begin
      Mouse := MousePresent;
      Keys  := Keys1
    end;
    with Device2 do begin
      Mouse := False;
      Keys  := Keys2
    end;
    ArkanType := T_SINGLE;
    MAspect   := 1
  end;
  with ConfigFile do begin
    CFG := Config;
    CRC := CalcCRC (Config)
  end;
  Write (f, ConfigFile);
  Close (f)
end;

procedure BBoosterApplication.ReadConfig;
var
  ConfigFile : BConfigFile;
  f          : file of BConfigFile;
begin
  Assign (f, CFG_FILE);
  Reset (f);
  if IOResult <> 0 then begin
    CreateConfig
  end else begin
    Read (f, ConfigFile);
    Close (f);
    with ConfigFile do
      if CRC <> CalcCRC (CFG) then
        CreateConfig
      else
        Config := CFG;
    with Config do begin
      with Device1 do
        Mouse := Mouse and MousePresent;
      with Device2 do
        Mouse := Mouse and MousePresent
    end
  end
end;

procedure BBoosterApplication.WriteConfig;
var
  ConfigFile : BConfigFile;
  f          : file of BConfigFile;
begin
  Assign (f, CFG_FILE);
  Rewrite (f);
  with ConfigFile do begin
    CFG := Config;
    CRC := CalcCRC (CFG)
  end;
  Write (f, ConfigFile);
  Close (f)
end;

function BBoosterApplication.GetPlace (sc : longint) : integer;
var
  i, j : integer;
begin
  with Config do begin
    i := 1;
    while (Tables [ArkanType, i].Score > sc) and (i <= MAX_SCORES) do Inc (i);
    GetPlace := 0;
    if i <= MAX_SCORES then begin
      GetPlace := i;
      for j := MAX_SCORES - 1 downto i do
        Tables [ArkanType, j+1] := Tables [ArkanType, j];
      with Tables [ArkanType, i] do begin
        Name  := '_';
        Score := sc
      end
    end
  end
end;

procedure BBoosterApplication.HallOfFame (NewSc : longint);
var
  Stars     : BStarsPtr;
  Color, i  : integer;
  NewPlace  : integer;
  CurPos    : integer;
  Time      : longint;
  EnterFlag : boolean;

  procedure DrawScore (n, color : integer);
  var
    st   : string;
    v, i : longint;
  begin
    st := '';
    with Config.Tables [Config.ArkanType, n] do begin
      st := st + Name;
      for i := 1 to NAME_LENGTH-Length (st) do
	st  := st + Chr (1);
      st := st + ' ';
      v := Score;
      for i := 0 to MAX_DIGITS do begin
	Insert (Chr (48+v mod 10), st, 42);
	v := v div 10
      end
    end;
    WriteString (120, 110+n*16, color, st)
  end;

  procedure NextColor;
  begin
    Color := Color * 2;
    if Color > 4 then
      Color := 1
  end;

  procedure EnterName;
  var
    ch : char;
  begin
    if AnyKey then begin
      ch := ReadKey;
      DrawScore (NewPlace, LightGray);
      with Config.Tables [Config.ArkanType, NewPlace] do begin
        Delete (Name, CurPos, 1);
        if KeyCurrent (LeftArrow) and (CurPos > 1) then
          Dec (CurPos);
        if KeyCurrent (RightArrow) and (CurPos <= Length (Name)) then
          Inc (CurPos);
        if KeyCurrent (Home) then
          CurPos := 1;
        if KeyCurrent (EndK) then
          CurPos := Length (Name)+1;
        if KeyCurrent (DeleteK) then
          Delete (Name, CurPos, 1);
        if KeyCurrent (BackSpace) and (CurPos > 1) then begin
          Delete (Name, CurPos - 1, 1);
          Dec (CurPos)
        end;
        if (ch <> #0) and (Length (Name)+1 < NAME_LENGTH) then begin
          Insert (ch, Name, CurPos);
          Inc (CurPos)
        end;
        if not KeyCurrent (Enter) then
          Insert ('_', Name, CurPos)
        else
          EnterFlag := False;
        DrawScore (NewPlace, LightGray)
      end
    end
  end;

begin
  ClearDevice;
  SetCharStyle (2, 1, 1);
  WriteString (208, 60, LightGreen, 'Hall Of Fame');
  SetCharStyle (0, 0, 0);
  NewPlace := GetPlace (NewSc);
  if NewPlace <> 0 then begin
    SetFillStyle (SolidFill, DarkGray);
    Bar (110, 105+NewPlace*16, 515, 120+NewPlace*16);
    SetColor (LightGray);
    Rectangle (110, 105+NewPlace*16, 515, 120+NewPlace*16)
  end;
  for i := 1 to MAX_SCORES do
    if i <> NewPlace then
      DrawScore (i, ColorData [(i-1) mod 6])
    else
      DrawScore (i, LightGray);
  New (Stars, Init (30));
  Color     := 1;
  CurPos    := 1;
  Time      := 0;
  EnterFlag := NewPlace <> 0;
  repeat
    if Time mod 30 = 0 then
      Stars^.Live;
    if Time mod 150 = 0 then begin
      for i := 1 to MAX_SCORES do begin
        if i <> NewPlace then
          DrawScore (i, Color);
        NextColor
      end;
      NextColor
    end;
    Delay (1);
    if (NewPlace <> 0) and EnterFlag then
      EnterName;
    Inc (Time)
  until (AnyKey or (GetShellEvent = cmFire)) and not EnterFlag;
  Dispose (Stars)
end;

constructor BBoosterApplication.Init;
begin
  OpenGraph (EGAHi, '');
  OpenGraphLibrary (GrLib, 'bbooster.bgl');
  SyncroInit (BBOOSTER_CPS);
  ShellInit;
  LoadCharSet ('bbooster.chr');
  ReadConfig;
  Randomize
end;

destructor BBoosterApplication.Done;
begin
  WriteConfig;
  FreeCharSet;
  ShellDone;
  CloseGraphLibrary (GrLib);
  CloseGraph
end;

procedure BBoosterApplication.QuitConfirm;
begin
  HalfFill (0, 0, GetMaxX, GetMaxY, Black);
  InitArea (2);
  Window3D (200, 155, 430, 200, 3, UP_DIR, '');
  SetCharStyle (0, 0, 0);
  WriteString (264, 175, LightBlue, 'REALLY QUIT ?');
  AddSprButton (cmQuit, 220, 172, GrLib, 'OK');
  AddSprButton (cmCancel, 395, 172, GrLib, 'CANCEL');
  ShowCursor;
  WaitEvent (StandardKeys);
  HideCursor
end;

procedure BBoosterApplication.Options;
var
  Cfg : BConfig;

  procedure DrawSlider;
  var
    x : integer;
  begin
    SetFillStyle (SolidFill, LightGray);
    Bar (205, 108, 435, 122);
    Line3D (207, 115, 429, 115, DOWN_DIR);
    x := 181+round ((Cfg.MAspect-0.5)*270);
      Window3D (x, 110, x+5, 120, 1, UP_DIR, '')
  end;

  procedure DrawMode;
  var
    Clr : byte;
  begin
    if Cfg.ArkanType = T_SINGLE then
      Clr := LightBlue
    else
      Clr := Blue;
    WriteString (216, 153, Clr, 'SINGLE');
    WriteString (360, 153, Clr xor $8, 'COOPERATIVE');
  end;

  procedure DrawDevice (x : integer; dvc : BDevice);
  begin
    if dvc.Mouse then
      PutSprite (GrLib, x, 195, 'MOUSE', NormalPut)
    else
      PutSprite (GrLib, x, 195, 'KEYBOARD', NormalPut)
  end;

  procedure DrawKey (x, y, color, key : integer; Text : string);
  var
    st : string;
    i  : integer;
  begin
    st := Text;
    for i := 1 to 15-Length (st+GetKeyString (key)) do
      st := st + chr (1);
    st := st + GetKeyString (key);
    WriteString (x+8, 232+y*10, color, st);
  end;

  procedure DrawKeys (x : integer; keys : TKeys);
  begin
    Frame3D (x, 230, x+129, 260, 3, DOWN_DIR);
    SetFillStyle (SolidFill, White);
    Bar (x, 230, x+129, 260);
    SetCharStyle (0, 0, 0);
    DrawKey (x, 0, Brown, Keys [LEFT], 'Left');
    DrawKey (x, 1, Green, Keys [RIGHT], 'Right');
    DrawKey (x, 2, Cyan, Keys [FIRE], 'Fire')
  end;

  procedure RedefineKeys (x : integer; var keys, check : TKeys);
  var
    i : TKeyType;

    procedure RedefineKey (y, Color : integer; var Key : byte; Text : string);
    var
      k   : byte;
      Rep : boolean;
      i   : TKeyType;
    begin
      DrawKey (x, y, Color, 0, Text);
      repeat
        repeat until AnyKey;
        k     := WhatPressed;
        Rep   := False;
        for i := LEFT to FIRE do
          Rep := Rep or (Keys [i] = k) or (Check [i] = k)
      until (GetKeyString (k) <> '') and not Rep;
      DrawKey (x, y, Color, 0, Text);
      DrawKey (x, y, Color, k, Text);
      Key := k
    end;

  begin
    Frame3D (x, 230, x+129, 260, 3, DOWN_DIR);
    SetFillStyle (SolidFill, White);
    Bar (x, 230, x+129, 260);
    SetCharStyle (0, 0, 0);
    for i := LEFT to FIRE do
      Keys [i] := 255;
    RedefineKey (0, Brown, Keys [LEFT], 'Left');
    RedefineKey (1, Green, Keys [RIGHT], 'Right');
    RedefineKey (2, Cyan,  Keys [FIRE], 'Fire')
  end;

begin
  Cfg := Config;
  ClearDevice;
  InitArea (2);
  Window3D (150, 50, 490, 300, 3, UP_DIR, '');
  SetCharStyle (2, 1, 0);
  XYText3D (260, 65, LightGray, UP_DIR, 'OPTIONS');
  SetCharStyle (0, 0, 0);
  WriteString (272, 95, LightMagenta, 'Sensetivity');
  AddButton (cmMSensDec, 171, 110, 15, 11);
  WriteString (176, 113, LightBlue, '<');
  AddButton (cmMSensInc, 451, 110, 15, 11);
  WriteString (456, 113, LightBlue, '>');
  WriteString (280, 130, LightRed, 'Game Mode');
  AddButton (cmSingle, 171, 150, 130, 11);
  AddButton (cmCooperative, 337, 150, 130, 11);
  WriteString (208, 175, Yellow, 'Player 1');
  WriteString (375, 175, Yellow, 'Player 2');
  AddButton (cmDevice1, 171, 195, 20, 14);
  DrawDevice (171, Cfg.Device1);
  AddSprButton (cmRedefine1, 282, 195, GrLib, 'REDEFINE');
  AddButton (cmDevice2, 337, 195, 20, 14);
  DrawDevice (337, Cfg.Device2);
  AddSprButton (cmRedefine2, 448, 195, GrLib, 'REDEFINE');
  DrawKeys (171, Cfg.Device1.Keys);
  DrawKeys (337, Cfg.Device2.Keys);
  DrawMode;
  DrawSlider;
  AddSprButton (cmEscape, 291, 275, GrLib, 'CANCEL');
  AddSprButton (cmOk    , 331, 275, GrLib, 'OK');
  repeat
    ShowCursor;
    WaitEvent (StandardKeys);
    HideCursor;
    with Cfg do
      case GetShellEvent of
        cmMSensDec : if MAspect > 0.6 then begin
          MAspect := MAspect - 0.1;
          DrawSlider
        end;
        cmMSensInc : if MAspect < 1.4 then begin
          MAspect := MAspect + 0.1;
          DrawSlider
        end;
        cmSingle : if ArkanType <> T_SINGLE then begin
          DrawMode;
          ArkanType := T_SINGLE;
          DrawMode
        end;
        cmCooperative : if ArkanType <> T_COOPERATIVE then begin
          DrawMode;
          ArkanType := T_COOPERATIVE;
          DrawMode
        end;
        cmDevice1 : with Device1 do if MousePresent then begin
          Mouse := not Mouse;
          DrawDevice (171, Device1);
          if Mouse then with Device2 do begin
            Mouse := Mouse and not Device1.Mouse;
            DrawDevice (337, Device2)
          end
        end;
        cmDevice2 : with Device2 do if MousePresent then begin
          Mouse := not Mouse;
          DrawDevice (337, Device2);
          if Mouse then with Device1 do begin
            Mouse := Mouse and not Device2.Mouse;
            DrawDevice (171, Device1)
          end
        end;
        cmRedefine1 : RedefineKeys (171, Device1.Keys, Device2.Keys);
        cmRedefine2 : RedefineKeys (337, Device2.Keys, Device1.Keys);
      end
  until (GetShellEvent = cmEscape) or (GetShellEvent = cmOk);
  if GetShellEvent = cmOk then
    Config := Cfg
end;

procedure BBoosterApplication.ShowMenu;
var
  i, j : integer;

  procedure DrawBall (x, y : integer);
  begin
    SetFillStyle (SolidFill, DarkGray);
    SetColor (DarkGray);
    PieSlice (x, y, 0, 360, 20);
    SetFillStyle (SolidFill, LightGray);
    SetColor (LightGray);
    PieSlice (x-3, y-3, 0, 360, 15);
    SetFillStyle (SolidFill, White);
    SetColor (White);
    PieSlice (x-5, y-5, 0, 360, 10)
  end;

begin
  ClearDevice;
  Window3D (185, 250, 446, 293, 3, UP_DIR, '');
  SetColor (White);
  SetCharStyle (10, 9, 1);
  DrawBall (100, 145);
  DrawBall (530, 145);
  WriteString (160, 60, LightRed, #5);
  WriteString (240, 50, Yellow, #6);
  WriteString (320, 50, LightGreen, #7);
  WriteString (400, 60, LightCyan, #7);
  SetCharStyle (6, 5, 1);
  WriteString (144, 150, LightBlue, #5#8#8#9#10#11#12);
  InitArea (5);
  AddSprButton (cmStartGame, 200, 260, GrLib, 'BUTTON1');
  AddSprButton (cmOptions, 250, 260, GrLib, 'BUTTON2');
  AddSprButton (cmHallOfFame, 300, 260, GrLib, 'BUTTON3');
  AddSprButton (cmHelp, 350, 260, GrLib, 'BUTTON4');
  AddSprButton (cmEscape, 400, 260, GrLib, 'BUTTON5');
  ShowCursor;
  WaitEvent (StandardKeys);
  HideCursor
end;

procedure BBoosterApplication.Run;
var
  Arkan : BArkanApplicationPtr;
begin
  repeat
    ShowMenu;
    case GetShellEvent of
      cmStartGame  : begin
        with Config do begin
          SetMouseAspect (MAspect);
          New (Arkan, Init (GrLib, Device1, Device2, ArkanType))
        end;
        with Arkan^ do begin
          Run;
          if GetArkanEvent = EV_GAMEOVER then
            HallOfFame (GetArkanScore)
        end;
        Dispose (Arkan, Done)
      end;
      cmOptions    : Options;
      cmHallOfFame : HallOfFame (0);
      cmHelp       : ToHelp (GrLib, HELP_FILE);
      cmEscape     : QuitConfirm;
    end
  until GetShellEvent = cmQuit
end;

begin
  New (BoosterApplication, Init);
  BoosterApplication^.Run;
  Dispose (BoosterApplication, Done)
end.
