uses Crt, Graph, Imager;

{$I-}

const
  FIELD_HGT = 22;
  FIELD_WDT = 19;

  BRK_WDT = 30;
  BRK_HGT = 12;

  L_MARGIN = 20;
  R_MARGIN = L_MARGIN+FIELD_WDT*BRK_WDT;
  T_MARGIN = 15;
  B_MARGIN = T_MARGIN+FIELD_HGT*BRK_HGT;

  MAX_LEVELS = 99;

  BackSpace  = $0008;
  Tab        = $0009;
  Enter      = $000D;
  Space      = $0020;
  F1         = $3B00;
  F9         = $4300;
  Home       = $4700;
  UpArrow    = $4800;
  PageUp     = $4900;
  LeftArrow  = $4B00;
  RightArrow = $4D00;
  EndK       = $4F00;
  DownArrow  = $5000;
  PageDown   = $5100;
  InsertK    = $5200;
  DeleteK    = $5300;
  Escape     = $001B;

type
  BBrickType  = (T_EMPTY, T_ONCE, T_TWICE, T_THREE, T_MULTIPLE, T_GOLDED,
                 T_EXPLODED, T_RESTORED, T_MOVED);

  BBrick = record
    BrkType : BBrickType;
    Color   : byte;
  end;

  BPackedLevel = record
    Name  : string [30];
    Level : array [0..FIELD_HGT, 0..FIELD_WDT] of byte;
  end;

  BLevel         = array [0..FIELD_HGT, 0..FIELD_WDT] of BBrick;

  BLevelsLibrary = array [0..MAX_LEVELS] of BPackedLevel;

var
  f         : TGrLib;
  Lib       : BLevelsLibrary;
  lvl       : BLevel;
  NumLevels : integer;
  CurLevel  : integer;
  CurColor  : integer;
  CurName   : string [30];
  CurType   : BBrickType;
  x, y      : integer;
  PenDown   : boolean;

procedure LoadLevelsLibrary (filename : string);
var
  f       : file of BPackedLevel;
  i, x, y : integer;
begin
  Assign (f, filename);
  Reset (f);
  NumLevels := 0;
  for i := 0 to MAX_LEVELS do
    for y := 0 to FIELD_HGT do
      for x := 0 to FIELD_WDT do
        Lib [i].Level [y, x] := 0;
  if IOResult = 0 then
    while not EOF (f) do begin
      Read (f, Lib [NumLevels]);
      Inc (NumLevels)
    end;
  Close (f)
end;

function IfEmptyLevel (var l : BPackedLevel) : boolean;
var
  res  : boolean;
  i, j : integer;
begin
  res := True;
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do
      if l.Level [i, j] <> 0 then begin
        res := False;
        Break
      end;
  IfEmptyLevel := res
end;

procedure SaveLevelsLibrary (filename : string);
var
  f : file of BPackedLevel;
  i : integer;
begin
  Assign (f, filename);
  Rewrite (f);
  for i := 0 to MAX_LEVELS do
    if not IfEmptyLevel (Lib [i]) then
      Write (f, Lib [i])
    else
      Break;
  Close (f)
end;

procedure StatusLine;
var
  s : string;
begin
  SetFillStyle (SolidFill, 0);
  Bar (0, GetMaxY-30, GetMaxX, GetMaxY);
  SetTextStyle (DefaultFont, HorizDir, 1);
  SetColor (Yellow);
  SetTextJustify (CenterText, TopText);
  OutTextXY (GetMaxX div 2, GetMaxY-25, CurName);
  SetTextJustify (LeftText, TopText);
  SetColor (LightGreen);
  OutTextXY (0, GetMaxY-10, 'Current level : ' + str (CurLevel+1));
  SetColor (White);
  OutTextXY (185, GetMaxY-10, 'Color : ');
  PutSprite (f, 250, GetMaxY-12, 'ABRICK' + Str (CurColor), XorPut);
  SetColor (LightCyan);
  OutTextXY (300, GetMaxY-10, 'X : '+str (x)+' Y : '+str (y));
  SetColor (LightRed);
  case CurType of
    T_EMPTY    : s := 'Empty';
    T_ONCE     : s := 'Single';
    T_TWICE    : if CurColor = 0 then
                   s := 'Glass'
                 else
		   s := 'Double';
    T_THREE    : s := 'Triple';
    T_MULTIPLE : s := 'Armoured';
    T_EXPLODED : s := 'Explosive';
    T_RESTORED : s := 'Restoring';
    T_GOLDED   : s := 'Gilded';
    T_MOVED    : s := 'Moved';
  end;
  OutTextXY (420, GetMaxY-10, 'Type : ' + s);
  SetColor (LightMagenta);
  if PenDown then
    OutTextXY (560, GetMaxY-10, 'Pen down')
  else
    OutTextXY (560, GetMaxY-10, 'Pen up')
end;

function BrkX (i : integer) : integer;
begin
  BrkX := L_MARGIN + i * BRK_WDT
end;

function BrkY (i : integer) : integer;
begin
  BrkY := T_MARGIN + i * BRK_HGT
end;

procedure DrawBrick (i, j : integer);
var
  x, y : integer;
  Brk  : BBrick;
begin
  x   := BrkX (j);
  y   := BrkY (i);
  Brk := Lvl [i, j];
  with Brk do
    case Brktype of
      T_EMPTY    : PutSprite (f, x, y, 'BREC', NormalPut);
      T_ONCE     : PutSprite (f, x, y, 'ABRICK'+str (Color), NormalPut);
      T_TWICE    : if Color = 0 then
                     PutSprite (f, x, y, 'ABRICK0', NormalPut)
                   else
		     PutSprite (f, x, y, 'BBRICK'+str (Color), NormalPut);
      T_THREE,
      T_MULTIPLE : PutSprite (f, x, y, 'CBRICK'+str (Color), NormalPut);

      T_EXPLODED : PutSprite (f, x, y, 'BEXPLODE', NormalPut);
      T_RESTORED : PutSprite (f, x, y, 'BRSTRD', NormalPut);
      T_GOLDED   : PutSprite (f, x, y, 'BGOLDED', NormalPut);
      T_MOVED    : PutSprite (f, x, y, 'BMOVED', NormalPut);
    end
end;

procedure Cursor;
begin
  PutSprite (f, BrkX (x)+11, BrkY (y)+3, 'XBALL', XorPut)
end;

procedure MoveCursor (dx, dy : integer);
begin
  Cursor;
  if (x+dx <= FIELD_WDT) and (x+dx >= 0) then
    Inc (x, dx);
  if (y+dy <= FIELD_HGT) and (y+dy >= 0) then
    Inc (y, dy);
  Cursor
end;

procedure Grid;
var
  i, j : integer;
begin
  SetColor (LightGray);
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do
      Rectangle (BrkX (j), BrkY (i), BrkX (j+1), BrkY (i+1));
end;

procedure UnpackLevel (PackedLevel : BPackedLevel);
var
  i, j : integer;
  Brk  : BBrickType;
begin
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do begin
      Lvl [i, j].Color    := PackedLevel.Level [i, j] and 7;
      Brk                 := BBrickType ((PackedLevel.Level [i, j] and $F8) shr 3);
      if Brk <= T_MOVED then
        Lvl [i, j].BrkType := Brk
      else
        Lvl [i, j].BrkType := T_EMPTY
    end;
  CurName := PackedLevel.Name
end;

procedure PackLevel (var PackedLevel : BPackedLevel);
var
  i, j : integer;
  tb   : byte;
begin
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do begin
      tb := 0;
      tb := Lvl [i, j].Color;
      tb := tb or (ord (Lvl [i, j].BrkType) shl 3);
      PackedLevel.Level [i, j] := tb
    end;
  PackedLevel.Name := CurName
end;

procedure DrawLevel;
var
  i, j : integer;
begin
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do
      DrawBrick (i, j)
end;


function EnterName : string;
const
  EWX1 = 200;
  EWY1 = 320;
  EWX2 = 440;
  EWY2 = 336;
  EWBKCLR = Black;
  EWFGCLR = Yellow;

var
  key           : word;
  curpos        : byte;
  c1, c2        : char;
  stname        : string;
  x1            : integer;
  y1            : integer;

  procedure PutCursor (pos : byte);
  var
    xx : word;
  begin
    xx := x1 + TextWidth (Copy (stname, 1, pos - 1));
    SetWriteMode (XorPut);
    SetColor (White);
    Line (xx-3, y1-2, xx+3, y1-2);
    Line (xx-3, y1+10, xx+3, y1+10);
    Line (xx, y1-2, xx, y1+10);
    SetWriteMode (NormalPut)
  end;

  procedure Text (index1, index2, color : byte);
  begin
    SetColor (color);
    OutTextXY (x1 + TextWidth (Copy (stname, 1, index1 - 1)), y1,
                             Copy (stname, index1, index2 - index1 + 1));
  end;

begin
  SetFillStyle (SolidFill, EWBKCLR);
  Bar (EWX1, EWY1, EWX2, EWY2);
  SetTextStyle (DefaultFont, HorizDir, 1);
  x1 := EWX1 + 10;
  y1 := EWY2 - 15;
  curpos := 1;
  stname := '';
  PutCursor (curpos);
  repeat
    if KeyPressed then begin
      PutCursor (curpos);
      c2 := #0;
      c1 := Readkey;
      if (c1 = #0) then
        c2 := Readkey;
      Key := ord (c2) * $100 + ord (c1);
      if (c1 >= #32)  and (TextWidth (stname+c1) < (EWX2-EWX1 - 10)) then
        begin
          Text (curpos, Length (stname), EWBKCLR);
          Insert (c1, stname, curpos);
          Text (curpos, Length (stname), EWFGCLR);
          Inc (curpos)
        end;
      if (Key = Backspace) and (curpos <> 1) then begin
        Text (curpos - 1, length (stname), EWBKCLR);
        Delete (stname, curpos - 1, 1);
        Text (curpos - 1, length (stname), EWFGCLR);
        Dec (curpos)
      end;
      if Key = Home then curpos := 1;
      if Key = EndK then curpos := Length (stname) + 1;
      if (Key = LeftArrow) and (curpos <> 1) then Dec (curpos);
      if (Key = RightArrow) and (curpos <> Length (stname) + 1) then Inc (curpos);
      if (Key = DeleteK) and (curpos <> Length (stname) + 1) then begin
        Text (curpos, length (stname), EWBKCLR);
        Delete (stname, curpos, 1);
        Text (curpos, length (stname), EWFGCLR)
      end;
      PutCursor (curpos);
    end
  until key = Enter;
  EnterName := stname
end;

procedure Editor;
var
  key     : char;
begin
  CurLevel := 0;
  CurColor := 1;
  x        := 0;
  y        := 0;
  PenDown  := False;
  CurType  := T_ONCE;
  Grid;
  UnpackLevel (Lib [CurLevel]);
  StatusLine;
  DrawLevel;
  Cursor;
  repeat
    Key := ReadKey;
    if Key = #0 then
      Key := ReadKey;
    case Key of
      '1'..'7' : if (CurType <> T_MULTIPLE) and (CurType <> T_GOLDED) and
                 ((CurType <> T_THREE) or (Key <> '6')) then
                   CurColor := ord (Key) - ord ('1') + 1;
      'q' : CurType := T_EMPTY;
      'w' : CurType := T_EXPLODED;
      'e' : CurType := T_RESTORED;
      'r' : begin
              CurType  := T_TWICE;
              CurColor := 0
            end;
      't' : begin
              CurType  := T_GOLDED;
              CurColor := 6
            end;
      'y' : begin
              CurType  := T_MOVED;
              CurColor := 2;
            end;
      'a' : CurType := T_ONCE;
      's' : CurType := T_TWICE;
      'd' : CurType := T_THREE;
      'f' : begin
              CurType  := T_MULTIPLE;
              CurColor := 6
            end;
      'n' : CurName := EnterName;
      #75 : MoveCursor (-1, 0);
      #77 : MoveCursor (1, 0);
      #72 : MoveCursor (0, -1);
      #80 : MoveCursor (0, 1);
      #13 : PenDown := not PenDown;
      '+' : if CurLevel < MAX_LEVELS then begin
              PackLevel (Lib [CurLevel]);
              Inc (CurLevel);
              UnpackLevel (Lib [CurLevel]);
              DrawLevel;
              Cursor
            end;
      '-' : if CurLevel > 0 then begin
              PackLevel (Lib [CurLevel]);
              Dec (CurLevel);
              UnpackLevel (Lib [CurLevel]);
              DrawLevel;
              Cursor
            end;
    end;
    if (key = ' ') or PenDown then begin
      if (lvl [y, x].color = CurColor) and
         (lvl [y, x].Brktype = CurType) then
           lvl [y, x].Brktype := T_EMPTY
        else begin
          lvl [y, x].color := CurColor;
          lvl [y, x].Brktype := CurType
        end;
      DrawBrick (y, x);
      Cursor
    end;
    if (CurType <> T_TWICE) and (CurColor = 0) then
      CurColor := 1;
    if (CurType = T_THREE) and (CurColor = 6) then
      CurColor := 1;
    StatusLine;
  until key = #27;
  PackLevel (Lib [CurLevel])
end;

begin
  OpenGraph (EGAHi, '');
  OpenGraphLibrary (f, 'bbooster.bgl');
  LoadLevelsLibrary ('bbooster.lvl');
  Editor;
  SaveLevelsLibrary ('bbooster.lvl');
  CloseGraphLibrary (f);
  CloseGraph
end.
