unit Help;

interface

  uses Graph, Imager, Shell, CharSet;

  Procedure XYText3D (x, y, color : integer; dir : Dir3D; text : string);
  Procedure ToHelp (var f : TGrLib; FileName : string);

implementation

const
  cmPrev = 1;
  cmNext = 2;

var
  MaxPage : integer;

procedure XYText3D (x, y, color : integer; dir : Dir3D; text : string);
begin
  if dir = UP_DIR then
    WriteString (x-1, y-1, UpperColor, text)
  else
    WriteString (x+1, y+1, UpperColor, text);
  if dir = DOWN_DIR then
    WriteString (x-1, y-1, BottomColor, text)
  else
    WriteString (x+1, y+1, BottomColor, text);
  WriteString (x, y, color, text)
end;

procedure DrawStars;
var
  i : integer;
begin
  Randomize;
  SetCharStyle (0, 0, 0);
  for i := 0 to 50 do
    WriteString (random (80)*8, random (GetMaxY)+2048,
		 random (8)+8, chr (random (4)+1))
end;

procedure DrawPage (var f : TGrLib; FileName : string; Page : integer);
var
  hlp       : text;
  pos       : integer;
  st        : string;
  cmd, parm : string;
  x, y      : integer;
  color     : word;
  i, j      : integer;
  Screen    : array [0..39, 0..18] of boolean;
  Image     : TImagePtr;

  function GetChar : char;
  var
    Res : char;
  begin
    Res := #0;
    repeat
      if pos > length (st) then begin
	Readln (hlp, st);
	Pos := 1
      end else begin
        Res := st [pos];
        Inc (Pos)
      end
    until Res >= #32;
    GetChar := Res
  end;

  procedure GetCommand (var cmnd, parm : string);
  var
    ch : char;
  begin
    cmnd := '';
    parm := '';
    ch   := GetChar;
    if ch = '<' then begin
      while (ch <> ' ') and (ch <> '>') do begin
	ch := GetChar;
        if (ch <> ' ') and (ch <> '>') then
          cmnd := cmnd + ch
      end;
      while ch <> '>' do begin
        ch := GetChar;
        if ch <> '>' then
          parm := parm + ch
      end
    end else
      parm := ch
  end;

begin
  SetCharStyle (0, 0, 0);
  SetFillStyle (SolidFill, Blue);
  Bar (150, 90, 490, 260);
  Assign (hlp, FileName);
  Reset (hlp);
  Readln (hlp, st);
  y     := 0;
  x     := 0;
  pos   := 1;
  Color := LightGreen;
  for i := 0 to 18 do
    for j := 0 to 39 do
      Screen [j, i] := True;
  repeat
    GetCommand (cmd, parm);
    if cmd <> 'EOF' then
      if cmd = 'CR' then
        repeat
	  x := 0;
          Inc (y);
          while not Screen [x, y] do
            Inc (x)
        until x < 40
      else
        if cmd = 'PAGE' then
          while ((cmd <> 'PAGE') or (parm <> str (page))) and (cmd <> 'EOF') do
	    GetCommand (cmd, parm)
        else
	  if cmd = '\PAGE' then
	    cmd := 'EOF'
	  else
	    if cmd = 'COLOR' then
	      Color := Val (Parm) xor Blue
	    else
	      if cmd = 'TITLE' then begin
                 SetCharStyle (2, 1, 1);
                 WriteString (160+(x+1)*8, 100+y*8, Color xor Blue, parm);
                 j := GetStringWidth (parm) div 8;
                 for i := x to x+j do begin
                   Screen [i, y]   := False;
                   Screen [i, y+1] := False
                 end;
                 Inc (x, j+1);
                 SetCharStyle (0, 0, 0)
	      end else
	        if cmd = 'SPRITE' then begin
		  LoadSprite (f, Image, parm);
                  PutImage ((20+x)*8, 100+y*8, Image, NormalPut);
                  for i := y to y+GetImageHgt (Image) div 8 do
                    for j := x to x+GetImageWdt (Image) div 8 do
                      Screen [j, i] := False;
                  x := j + 1;
                  FreeImage (Image)
	      end else
	        if cmd = 'SETX' then
		  x := Val (parm)
		else
		  if cmd = 'SETY' then
		    y := Val (parm)
		  else
		    if cmd = 'MAXPAGE' then
		      MaxPage := Val (Parm)
		    else begin
		      repeat
			if not Screen [x, y] then
			  Inc (x);
			if x >= 40 then begin
			  x := 0;
			  Inc (y)
			end;
		      until Screen [x, y];
		      WriteString (160+x*8, 2148+y*8, Color, parm);
		      Screen [x, y] := False;
		      Inc (x)
		    end;
  until cmd = 'EOF';
  Close (hlp)
end;

procedure ToHelp (var f : TGrLib; FileName : string);
var
  Page  : integer;
  i     : integer;

  procedure PageIndicator;
  var
    Color : integer;
  begin
    SetCharStyle (0, 0, 0);
    WriteString (296, 2316, 3, Str (Page+1)+'/'+Str (MaxPage+1));
    if Page = 0 then
      Color := Magenta
    else
      Color := LightMagenta;
    WriteString (120, 2224, Color, '<');
    if Page = MaxPage then
      Color := Magenta
    else
      Color := LightMagenta;
    WriteString (512, 2224, Color, '>');
  end;

begin
  SetVisualPage (0);
  SetActivePage (1);
  SetMousePage (1);
  ClearDevice;
  DrawStars;
  SetWriteMode (NormalPut);
  InitArea (1);
  Window3D (100, 70, 540, 280, 3, UP_DIR, '');
  for i := 0 to 2 do
    Line3D (99, 110+i*10, 540, 110+i*10, DOWN_DIR);
  Frame3D (150, 90, 490, 260, 3, DOWN_DIR);
  Page := 0;
  DrawPage (f, filename, Page);
  AddButton (cmPrev, 118, 160, 10, 40);
  AddButton (cmNext, 510, 160, 10, 40);
  PageIndicator;
  SetVisualPage (1);
  repeat
    ShowCursor;
    WaitEvent (StandardKeys);
    HideCursor;
    case GetShellEvent of
      cmPrev : if Page > 0 then begin
		 PageIndicator;
		 Dec (Page);
		 PageIndicator;
		 DrawPage (f, filename, Page)
	       end;
      cmNext : if Page < MaxPage then begin
		 PageIndicator;
		 Inc (Page);
		 PageIndicator;
		 DrawPage (f, filename, Page)
	       end
    end
  until GetShellEvent = cmEscape;
  SetActivePage (0);
  SetVisualPage (0);
  SetMousePage (0);
  ClearShellEvent
end;

begin
end.
