unit Imager;

{
     'Imager' ।祭  ࠡ  ᪨ ⥪,
    ᮤঠ騬  ᥡ ࠩ,  ᫥⢨  ᯮ짮 -
    楤ன PutImage.  ᮤন ⠪ 楤  㭪樨, 騥
    ࠡ  䨪, ࠡ  ⨯஢묨 䠩  ..
}

interface

uses Graph, Dos, Crt;

type
  Dir3D     = (UP_DIR, DOWN_DIR);
  TOperType = (F_READ, F_WRITE);

  TImage = record
    Wdt, Hgt : word;
    Data     : array [0..65530] of byte
  end;
  TImagePtr = ^TImage;

  TSpriteHeader = record
    Name           : string [8];
    Offset, Length : longint;
    Wdt, Hgt       : word;
  end;
  TSpriteHeaderPtr = ^TSpriteHeader;

  TGrLib = record
    GrLibFile    : file;
    HeaderPtr    : TSpriteHeaderPtr;
    NumOfSprites : byte
  end;

const
  GrLibSignLen = 31;
  GrLibSign : string [GrLibSignLen] = 'BALL BOOSTER GRAPHIC LIBRARY ->';

  temp      : TImagePtr = NIL;

var
  OldName     : string[8];

  BottomColor : byte;
  MidColor    : byte;
  UpperColor  : byte;

procedure AllowInt; inline ($FB);
procedure ForbidInt; inline ($FA);
procedure Beep (Hz : word);
procedure Error (err : string);
function  Str (number : longint) : string;
function  Val (st : string) : longint;
function  StrUpCase (Str : string) : string;
function  GetFileName (var f : file) : string;
procedure FOpen (var f : file; FileName : string; OpenType : TOperType);
procedure BlockOper (var f : file; var data; count : word; OperType : TOperType);
procedure GetMem (var p; length : longint);

procedure OpenGraph (GraphMode : integer; Path : string);
procedure SetImageSize (Image : TImagePtr; NewWdt, NewHgt : word);
function  GetImageLength (x1, y1, x2, y2 : longint) : longint;
function  GetRealImageLength (Image : TImagePtr) : word;
function  GetImageWdt (Image : TImagePtr) : word;
function  GetImageHgt (Image : TImagePtr) : word;
procedure PutImage (x, y : integer; Image : TImagePtr; BitBlt : word);
procedure GetImage (x, y : integer; Image : TImagePtr);
procedure CreateImage (var Image : TImagePtr; Wdt, Hgt : word);
procedure FreeImage (var Image : TImagePtr);
procedure OpenGraphLibrary (var GrLib : TGrLib; FileName : string);
procedure CloseGraphLibrary (var GrLib : TGrLib);
procedure LoadSprite (var GrLib : TGrLib; var Image : TImagePtr; SpriteName : string);
procedure PutSprite (var GrLib : TGrLib; x, y : word; Name : string; BitBlt : word);
procedure CloseGraph;
procedure Set3DColors (clr1, clr2, clr3 : byte);
procedure Line3D (x1, y1, x2, y2 : word; dir : Dir3D);
procedure Frame3D (x1, y1, x2, y2 : word; ht : byte; dir : Dir3D);
procedure Window3D (x1, y1, x2, y2 : word; ht : byte;
                    dir : Dir3D; Header : string);
procedure Text3D (color : byte; dir : Dir3D; text : string);
procedure XYText3D (x, y : word; color : byte; dir : Dir3D; text : string);
procedure HalfFill (x1, y1, x2, y2 : integer; Color : byte);

implementation

procedure Beep (Hz : word);
begin
  Sound (Hz);
  Delay (10);
  Nosound
end;

procedure Error (err : string);
begin
  CloseGraph;
  ClrScr;
  TextColor (LightRed);
  Writeln ('RUNTIME ERROR !!! ');
  Writeln;
  TextColor (UpperColor);
  Writeln (err);
  InOutRes := 0;
  Halt (1)
end;

function Str (number : longint) : string;
var
  st : string;
begin
  System.Str (number, st);
  Str := st
end;

function Val (st : string) : longint;
var
  res  : longint;
  code : integer;
begin
  System.Val (st, res, code);
  if Code <> 0 then
    Error ('Function Val crashed !');
  Val := res
end;

function StrUpCase (Str : string) : string;
var
  NewStr : string;
  i     : byte;
begin
  NewStr := '';
  for i := 1 to Length (Str) do
    NewStr := NewStr + UpCase (Str [i]);
  StrUpCase := NewStr
end;

function GetFileName (var f : file) : string;
var
  FileName : string;
  CharPtr  : ^char;
begin
  FileName := '';
  CharPtr := @FileRec (F).Name;
  while CharPtr^ <> #0 do begin
    FileName := FileName + CharPtr^;
    Inc (CharPtr)
  end;
  GetFileName := FileName
end;

procedure FOpen (var f : file; FileName : string; OpenType : TOperType);
var
  TypeStr : string;
begin
  Assign (f, FileName);
  case OpenType of
    F_READ  :
              begin
                Reset (f, 1);
                TypeStr := 'reading.'
              end;
    F_WRITE :
              begin
                Rewrite (f, 1);
		TypeStr := 'writing.'
              end
  end;
  if IOResult <> 0 then
    Error ('Can''t open ' + FileName + ' for ' + TypeStr)
end;

procedure BlockOper (var f : file; var data; count : word; OperType : TOperType);
var
  BytesCount : word;
  TypeStr    : string;
begin
  case OperType of
    F_READ :
             begin
	       System.BlockRead (F, Data, Count, BytesCount);
	       TypeStr := 'read from '
             end;
    F_WRITE :
              begin
                System.BlockWrite (F, Data, Count, BytesCount);
                TypeStr := 'write to '
              end
  end;
  if (IOResult <> 0) or (Count <> BytesCount) then begin
    Close (f);
    Error ('Can''t ' + TypeStr + GetFileName (f))
  end
end;

procedure GetMem (var p; length : longint);
begin
  if (length > 65528) then
    Error ('Too big length of allocated block on heap in GetMem.')
  else
    if MaxAvail < length then
      Error ('Can''t allocate ' + Str (length) + ' bytes.')
    else
      System.GetMem (pointer (p), length)
end;

procedure EGAVGADriver; external;
  {$L egavga.obj}

procedure OpenGraph (GraphMode : integer; Path : string);
var
  GraphDriver, Result : integer;
begin
  GraphDriver := RegisterBGIDriver (@EGAVGADriver);
  if GraphDriver < 0 then
    Error ('Can''t register graphic driver !');
  GraphDriver := EGA;
  InitGraph (GraphDriver, GraphMode, Path);
  Result := GraphResult;
  if Result <> grOk then
    Error ('Can''t init graphic driver : ' + GraphErrorMSG (result))
end;

procedure SetImageSize (Image : TImagePtr; NewWdt, NewHgt : word);
begin
  with Image^ do begin
    Wdt := NewWdt;
    Hgt := NewHgt
  end
end;

function GetImageLength (x1, y1, x2, y2 : longint) : longint;
begin
  GetImageLength := ImageSize (x1, y1, x2, y2)
end;

function GetRealImageLength (Image : TImagePtr) : word;
begin
  with Image^ do
    GetRealImageLength := GetImageLength (0, 0, Wdt, Hgt)
end;

function GetImageWdt (Image : TImagePtr) : word;
begin
  GetImageWdt := Image^.Wdt
end;

function GetImageHgt (Image : TImagePtr) : word;
begin
  GetImageHgt := Image^.Hgt
end;

procedure PutImage (x, y : integer; Image : TImagePtr; BitBlt : word);
begin
  Graph.PutImage (x, y, Image^, BitBlt)
end;

procedure GetImage (x, y : integer; Image : TImagePtr);
begin
  with Image^ do
    Graph.GetImage (x, y, x + Wdt, y + Hgt, Image^)
end;

procedure CreateImage (var Image : TImagePtr; Wdt, Hgt : word);
begin
  GetMem (Image, GetImageLength (0, 0, Wdt, Hgt));
  SetImageSize (Image, Wdt, Hgt)
end;

procedure FreeImage (var Image : TImagePtr);
begin
  if Image <> NIL then begin
    FreeMem (Image, GetRealImageLength (Image));
    Image := NIL
  end
end;

procedure OpenGraphLibrary (var GrLib : TGrLib; FileName : string);
var
  GrLibString : string [GrLibSignLen];
  HOFFS       : longint;
  HeaderSize  : longint;
begin
  with GrLib do begin
    FileName := StrUpCase (FileName);
    FOpen (GrLibFile, FileName, F_READ);
    BlockOper (GrLibFile, GrLibString, sizeof (GrLibString), F_READ);
    if GrLibString <> GrLibSign then
      Error ('Invalid graphic library ' + FileName);
    BlockOper (GrLibFile, HOFFS, sizeof (HOFFS), F_READ);
    Seek (GrLibFile, HOFFS);
    HeaderSize := FileSize (GrLibFile) - HOFFS;
        NumOfSprites := HeaderSize div sizeof (TSpriteHeader);
    GetMem (HeaderPtr, HeaderSize);
    BlockOper (GrLibFile, HeaderPtr^, HeaderSize, F_READ)
  end
end;

procedure CloseGraphLibrary (var GrLib : TGrLib);
begin
  with GrLib do begin
    FreeMem (HeaderPtr, NumOfSprites * sizeof (TSpriteHeader));
    Close (GrLibFile)
  end
end;

procedure LoadSprite (var GrLib : TGrLib; var Image : TImagePtr; SpriteName : string);
var
  OldHeaderPtr : TSpriteHeaderPtr;
  i            : longint;
begin
  with GrLib do begin
    OldHeaderPtr := HeaderPtr;
    i := 0;
    repeat
      if HeaderPtr^.Name = StrUpCase (SpriteName) then break;
      inc (HeaderPtr);
      inc (i)
    until i >= NumOfSprites;
    if i >= NumOfSprites then
      Error ('Can''t find sprite ' + SpriteName + ' in ' + GetFileName (GrLibFile));
    with HeaderPtr^ do begin
      CreateImage (Image, Wdt, Hgt);
      Seek (GrLibFile, Offset);
      BlockOper (GrLibFile, Image^.Data, Length, F_READ)
    end;
    HeaderPtr := OldHeaderPtr;
  end
end;

procedure PutSprite (var GrLib : TGrLib; x, y : word; Name : string; BitBlt : word);
begin
  if OldName <> Name then begin
    FreeImage (Temp);
    LoadSprite (GrLib, Temp, Name)
  end;
  PutImage (x, y, Temp, BitBlt);
  OldName := Name
end;

procedure CloseGraph;
begin
  Graph.CloseGraph
end;

procedure Set3DColors (clr1, clr2, clr3 : byte);
begin
  BottomColor := clr1;
  MidColor    := clr2;
  UpperColor  := clr3
end;

procedure Line3D (x1, y1, x2, y2 : word; dir : Dir3D);
begin
  if dir = UP_DIR then
    SetColor (UpperColor)
  else
    SetColor (BottomColor);
  Line (x1, y1, x2, y2);
  if dir = UP_DIR then
    SetColor (BottomColor)
  else
    SetColor (UpperColor);
  Line (x1+1, y1+1, x2+1, y2+1)
end;

procedure Frame3D (x1, y1, x2, y2 : word; ht : byte; dir : Dir3D);
var
  i : byte;
begin
  if dir = UP_DIR then
    SetColor (UpperColor)
  else
    SetColor (BottomColor);
  for i := 1 to ht do begin
    Line (x1-i, y1-i, x2+i, y1-i);
    Line (x1-i, y1-i, x1-i, y2+i)
  end;
  if dir = UP_DIR then
    SetColor (BottomColor)
  else
    SetColor (UpperColor);
  for i := 1 to ht do begin
    Line (x1-i, y2+i, x2+i, y2+i);
    Line (x2+i, y1-i, x2+i, y2+i)
  end
end;

procedure Window3D (x1, y1, x2, y2 : word; ht : byte;
                    dir : Dir3D; Header : string);
var
  i : byte;
begin
  SetFillStyle (SolidFill, MidColor);
  Bar (x1, y1, x2, y2);
  if Header <> '' then begin
    SetFillStyle (SolidFill, Blue);
    Bar (x1, y1, x2 - 2, y1 + 13);
    Frame3D (x1 + 2, y1 + 2, x2 - 2, y1 + 13, 1, DOWN_DIR);
    SetColor (UpperColor);
    SetTextJustify (CenterText, TopText);
    SetTextStyle (SmallFont, HorizDir, 4);
    OutTextXY((x1 + x2) div 2, y1 + 1, Header);
    SetTextJustify (LeftText, TopText)
  end;
  Frame3D (x1, y1, x2, y2, ht, dir)
end;

procedure Text3D (color : byte; dir : Dir3D; text : string);
var
  x, y : word;
begin
  x := GetX;
  y := GetY;
  SetColor (UpperColor);
  if dir = UP_DIR then begin
    MoveTo (x-1, y-1);
    OutText (text)
  end else begin
    MoveTo (x+1, y+1);
    OutText (text)
  end;
  SetColor (BottomColor);
  if dir = DOWN_DIR then begin
    MoveTo (x-1, y-1);
    OutTextXY (x-1, y-1, text)
  end else begin
    MoveTo (x+1, y+1);
    OutTextXY (x+1, y+1, text)
  end;
  SetColor (color);
  MoveTo (x, y);
  OutText (text)
end;

procedure XYText3D (x, y : word; color : byte; dir : Dir3D; text : string);
begin
  MoveTo (x, y);
  Text3D (color, dir, text);
end;

procedure HalfFill (x1, y1, x2, y2 : integer; Color : byte);
var
  i : word;
begin
  SetColor (Color);
  for i := y1 to y2 do begin
    SetLineStyle (UserBitLn, $5555 * (i mod 2 + 1), NormWidth);
    Line (x1, i, x2, i)
  end;
  SetLineStyle (SolidLn, 0, NormWidth)
end;

begin
  Set3DColors (DarkGray, LightGray, White)
end.
