program Images_Maker;

uses
  Dos,
  Crt;

{$I-}

const
  BGL_SIGN : string[31] = 'BALL BOOSTER GRAPHIC LIBRARY ->';
  INI_FILE = 'mkimages.ini';
  BUF_SIZE = 2048;

  LNum    : integer = 0;
  Buf1    : string  = '';
  Buf2    : string  = '';
  EOBFlag : boolean = False;
  EOFFlag : boolean = False;

type
  OpenType = (F_READ, F_WRITE);

  TImageDescriptor = record
    Sign   : string [8];
    Offset : longint;
    Size   : longint;
    Len    : word;
    Hgt    : word
  end;

  ImRecord = record
    Next  : pointer;
    Image : TImageDescriptor
  end;

  ImageHeader = record
    Len   : word;
    Hgt   : word
  end;

  CharPtr     = ^char;
  ImRecordPtr = ^ImRecord;

var
  OLn        : integer;
  FIni       : text;
  EOFF, EOBF : boolean;
  SrcDir     : string;
  DestDir    : string;
  Buffer     : array [1..BUF_SIZE] of byte;

  HOFFS      : longint;
  HeadPtr    : ImRecordPtr;

procedure Error (err : string);
begin
  TextColor (LightRed);
  Writeln;
  Writeln;
  Writeln ('IMAGES MAKER ERROR : ');
  Writeln;
  Writeln (err)
end;

function GetFileName (nchar : CharPtr) : string;
var
  stname : string;
begin
  stname := '';
  repeat
    stname := stname + nchar^;
    Inc (nchar)
  until nchar^ = #0;
  GetFileName := stname
end;

procedure BlockRead (var f : file; var data; count : word);
var
  BytesRead : word;
begin
  System.BlockRead (F, Data, Count, BytesRead);
  if (IOResult <> 0) or (Count <> BytesRead) then begin
    Error ('Can''t Read From ' + GetFileName (@FileRec (f).Name));
    Close (f);
    InOutRes := 0;
    Halt (1)
  end
end;

procedure BlockWrite (var f : file; var data; count : word);
var
  BytesRead : word;
begin
  System.BlockWrite (F, Data, Count, BytesRead);
  if (IOResult <> 0) or (Count <> BytesRead) then begin
    Error ('Can''t Write To ' + GetFileName (@FileRec (f).Name));
    Close (f);
    InOutRes := 0;
    Halt (1)
  end
end;

function StrC (n : longint) : string;
var
  st : string;
begin
  Str (n, st);
  StrC := st
end;

procedure NeededError (var f : text; par : string);
begin
  Error ('Needed ' + par + ' in ' + StrC (oln) + ' string of ' + GetFileName (@TextRec(f).Name) + '.');
  Close (f);
  InOutRes := 0;
  Halt (1)
end;

function SpKill (st : string) : string;
var
  i     : byte;
  newst : string;
  ch    : char;
begin
  newst := '';
  for i := 1 to Length (st) do begin
    ch := UpCase (st [i]);
    if ch <> ' ' then newst := newst + ch
  end;
  SpKill := newst
end;

procedure ReadLine (var f : text;var st : string);
begin
  Buf1 := Buf2;
  oln := lnum;
  EOFFlag := EOFF;
  EOBFlag := EOBF;
  repeat
    ReadLn (f, Buf2);
    Inc (lnum)
  until (SpKill (Buf2) <> '') or SeekEOF (f);
  if EOF (f) then begin
    EOFF := True;
    EOBF := True
  end else
    EOBF := False;
  st := Buf1;
  if (pos ('[', Buf2) > 0) then
    EOBFlag := True
end;

function SearchParameter (var f : text; dir : string) : string;
var
  st     : string;
  param  : string;
  stdir  : string;
  p      : byte;
  ln     : byte;
begin
  param := '';
  ReadLine (f, st);
  st  := SpKill (st);
  dir := SpKill (dir);
  p := pos ('=', st);
  if p > 0 then begin
    stdir := copy (st, 1, p - 1);
    if stdir = dir then begin
      param := copy (st, p + 1, Length (st) - p);
      if param = '' then
        NeededError (f, 'parameter in directive ''' + dir + ' = ''')
    end else
      NeededError (f, 'directive ''' + dir + ' = '' instead of ''' + stdir + ' = ''')
  end else
    NeededError (f, 'directive ''' + dir + '''');
  SearchParameter := param
end;

procedure SearchSection (var f : text; sect : string);
var
  st      : string;
  stsect  : string;
begin
  stsect := '';
  ReadLine (f, st);
  st   := SpKill (st);
  sect := SpKill (sect);
  if st [1] = '[' then
    if st [length (st)] = ']' then begin
      stsect := copy (st, 2, length (st) - 2);
      if stsect <> sect then
        NeededError (f, 'section [' + sect + '] instead of [' + stsect + ']')
    end else
      NeededError (f, '<]>')
  else
    NeededError (f, 'section [' + sect + ']')
end;

procedure OpenINI (var f : text; filename : string);
var
  st : string;
begin
  Assign (f, 'mkimages.ini');
  Reset  (f);
  if IOResult <> 0 then begin
    Error ('Can''t Open ' + filename);
    InOutRes := 0;
    Halt (1)
  end;
  ReadLine (f, st)
end;

procedure CopyBytes (var src, dest : file; size : longint);
var
  FillSize : longint;
begin
  repeat
    if size >= BUF_SIZE then
      FillSize := BUF_SIZE
    else
      FillSize := size;
    BlockRead (src, Buffer, FillSize);
    BlockWrite (dest, Buffer, FillSize);
    Write ('.');
    Dec (size, FillSize)
  until size = 0
end;

procedure FOpen (var f : file; filename : string; open : OpenType);
begin
  Assign (f, filename);
  if open = F_READ then
    Reset (f, 1)
  else Rewrite (f, 1);
  if IOResult <> 0 then begin
    Error ('Can''t open ' + filename);
    InOutRes := 0;
    Halt (1)
  end
end;

procedure ParamSeparator (var f : text; src : string;
                          var param1, param2 : string);
var
  SepPos : byte;
begin
  SepPos := Pos (',', src);
  if SepPos > 0 then begin
    param1 := Copy (src, 1, SepPos - 1);
    param2 := Copy (src, SepPos + 1, Length (src) - SepPos);
    if param1 = '' then
      NeededError (f, 'first parameter');
    if param2 = '' then
      NeededError (f, 'second parameter')
  end else
    NeededError (f, 'separator '',''')
end;

procedure AddImage (var f : file; parm : string);
var
  ImageName : string;
  Signature : string;
  ptr       : ImRecordPtr;
  ISize     : longint;
  Ratio     : single;
  FImage    : file;
  HImage    : ImageHeader;
begin
  ParamSeparator (FIni, parm, Signature, ImageName);
  Writeln;
  TextColor (LightCyan);
  Write ('Adding ');
  TextColor (White);
  Write (ImageName);
  TextColor (LightMagenta);
  GotoXY (30, WhereY);
  Write (Signature);
  GotoXY (38, WhereY);
  FOpen (FImage, SrcDir + ImageName, F_READ);
  ISize := FileSize (FImage) - sizeof (HImage);
  BlockRead (FImage, HImage, sizeof (HImage));
  GetMem (ptr, sizeof (ImRecord));
  ptr^.next := HeadPtr;
  HeadPtr := ptr;
  with ptr^.image do begin
    sign   := Signature;
    offset := FilePos (f);
    size   := ISize;
    len    := HImage.Len;
    hgt    := HImage.Hgt
  end;
  CopyBytes (FImage, f, ISize);
  Close (FImage);
  TextColor (LightGreen);
  Write (' ok')
end;

procedure SaveHeader (var f : file);
var
  ptr : ImRecordPtr;
begin
  TextColor (Yellow);
  Writeln;
  Writeln;
  Write ('Saving Header ');
  HOFFS := FilePos (f);
  while HeadPtr <> NIL do begin
    BlockWrite (f, HeadPtr^.image, sizeof (TImageDescriptor));
    Write ('.');
    ptr := HeadPtr^.Next;
    FreeMem (HeadPtr, sizeof (ImRecord));
    HeadPtr := ptr
  end;
  Seek (f, sizeof (BGL_SIGN));
  BlockWrite (f, HOFFS, sizeof (HOFFS));
  TextColor (LightGreen);
  Writeln (' ok !')
end;

procedure BuildBGL;
var
  FBgl : file;
  Name : string;
begin
  Writeln;
  SearchSection (FIni, 'GraphicLibrary');
  Name := SearchParameter (FIni, 'FileName');
  TextColor (LightBlue);
  Writeln ('Graphic Library Filename - ', Name);
  FOpen (FBgl, DestDir + Name, F_WRITE);
  BlockWrite (FBgl, BGL_SIGN, sizeof (BGL_SIGN));
  BlockWrite (FBgl, HOFFS, sizeof (HOFFS));
  HeadPtr := NIL;
  repeat
    AddImage (FBgl, SearchParameter (FIni, 'img'));
  until EOBFlag;
  SaveHeader (FBgl);
  Close (FBgl)
end;

begin
  ClrScr;
  TextColor (LightCyan);
  Writeln ('Images Maker for Ball Booster v1.0');
  Writeln;
  TextColor (White);
  Write ('  Opening ' + INI_FILE + '...');
  OpenINI (FIni, INI_FILE);
  TextColor (LightGreen);
  Writeln (' ok');
  SrcDir  := SearchParameter (FIni, 'Source_Directory');
  DestDir := SearchParameter (FIni, 'Destination_Directory');
  TextColor (LightCyan);
  Writeln;
  Write ('  Source Directory      - ');
  TextColor (White);
  Writeln (SrcDir);
  TextColor (LightCyan);
  Write ('  Destination Directory - ');
  TextColor (White);
  Writeln (DestDir);
  while not EOFFlag do begin
    BuildBGL;
    if (ParamCount = 1) and (SpKill (ParamStr (1)) = '/WAIT') then begin
      TextColor (LightGray);
      Writeln;
      Writeln ('Press Any Key...');
      repeat until KeyPressed;
      ReadKey
    end
  end;
  TextColor (LightCyan);
  Writeln;
  Writeln ('Building Finished Succesfully !');
  Close (FIni)
end.
