unit Objects;

interface

const
  FIELDS_FILE = 'bbooster.lvl';

  FIELD_HGT = 22;
  FIELD_WDT = 19;

  BRK_WDT = 30;
  BRK_HGT = 12;

  L_MARGIN = 20;
  R_MARGIN = 620;
  T_MARGIN = 0;
  B_MARGIN = 350;

  KAccel = 2000;
  KSpeed = 6;

type
  BReal       = Extended;

  BBrickType  = (T_EMPTY, T_ONCE, T_TWICE, T_THREE, T_MULTIPLE, T_GOLDED,
                 T_EXPLODED, T_RESTORED, T_MOVED);

  BPackedField = record
    Name  : string [30];
    Field : array [0..FIELD_HGT, 0..FIELD_WDT] of byte;
  end;

  BRealRect = record
    rx1, ry1, rx2, ry2 : BReal;
  end;

  BRect = record
    x1, y1, x2, y2 : integer;
  end;

  BDelta = record
    dx, dy, accel, speed : BReal;
  end;

  BBrick = record
    BrkType  : BBrickType;
    Color    : byte;
  end;

  BPrizeType = (T_ALLOFF, T_PDKILL, T_PDEXTRASHRINK, T_GRAVBALL, T_FASTBALL,
                T_PDSHRINK, T_SMALLBALL, T_PDEXPAND, T_SLOWBALL, T_MULTBALL,
                T_PDMAGNITY, T_PDSHOOTING, T_XSETOFF, T_ZAPBRICKS,
                T_MULTEXPLODING, T_BLEXPLODED, T_THRUBRICKS, T_EXTRABONUS,
                T_WARPLEVEL, T_EXTRALIFE, T_NONE);
  BPrizesSet = set of BPrizeType;

  BPrizeSetPtr = ^BPrizeSet;
  BPrizeSet    = object
    private
      AddedSet   : BPrizesSet;
      RemovedSet : BPrizesSet;
      PrizeSet   : BPrizesSet;
    public
      Constructor Init;
      Procedure   AddPrize (PrizeType : BPrizeType);
      Procedure   RemovePrize (PrizeType : BPrizeType);
      Procedure   ClearSet;
      Function    InPrizeSet (PrizeType : BPrizeType) : boolean;
      Function    InAddedSet (PrizeType : BPrizeType) : boolean;
      Function    InRemovedSet (PrizeType : BPrizeType) : boolean;
  end;

  BFieldPtr = ^BField;
  BField    = object
    private
      Field       : array [0..FIELD_HGT, 0..FIELD_WDT] of BBrick;
      F           : file of BPackedField;
      Name        : string;
      BricksCount : integer;
    public
      Constructor Init;
      Destructor  Done; virtual;
      Procedure   UnpackField (var PackedField : BPackedField);
      Procedure   SeekField (nf : integer);
      Procedure   LoadNextField;
      Procedure   GetBrick (i, j : integer; var Brk : BBrick);
      Procedure   SetBrick (i, j : integer; Brk : BBrick);
      Procedure   DrawBrick (i, j : integer); virtual;
      Procedure   DrawField;
      Procedure   IncBricksCount (dc : integer);
      Procedure   CollisionHandler (dlt : BDelta; i, j : integer;
                                    exploded, moved : boolean); virtual;
      Function    EOFields : boolean;
      Function    GetFieldName : string;
      Function    GetNumOfBricks : integer;
      Function    DetectCollision (dlt : BDelta; bx1, by1, bx2, by2 : integer;
                                   exploded : boolean) : boolean;
      Procedure   SetBricksCount (BrCount : integer);
      Function    GetBricksCount : integer;
  end;

  BObjectPtr = ^BObject;
  BObject    = object
    private
      BoundsOld   : BRect;
      Bounds      : BRect;
      DieFlag     : boolean;
      RefreshFlag : boolean;
    public
      Constructor Init (bx1, by1, bx2, by2 : integer);
      Destructor  Done; virtual;
      Procedure   ToLive (var bBounds : BRect); virtual;
      Procedure   ToDraw (x, y : integer); virtual;
      Procedure   ToDie; virtual;
      Procedure   SetBounds (bx1, by1, bx2, by2 : integer); virtual;
      Procedure   GetBounds (var bBounds : BRect);
      Procedure   Refresh;
      Function    GetX1     : integer;
      Function    GetY1     : integer;
      Function    GetX2     : integer;
      Function    GetY2     : integer;
  end;

  BAnimationPtr = ^BAnimation;
  BAnimation    = object (BObject)
    private
      Time   : integer;
      NFrame : integer;
      Frames : integer;
      Speed  : integer;
    public
      Constructor Init (i, j, bFrames, bSpeed : integer);
      Procedure   ToLive (var bBounds : BRect); virtual;
      Procedure   DrawFrame (x, y, n : integer); virtual;
      Destructor  Done; virtual;
  end;

  BGravityObjectPtr = ^BGravityObject;
  BGravityObject    = object (BObject)
    private
      RealBounds   : BRealRect;
      Delta        : BDelta;
      BoundsChange : boolean;
      DeltaChange  : boolean;
    public
      Constructor Init (bx1, by1, bx2, by2 : integer; bdlt : BDelta);
      Procedure   ToLive (var bBounds : BRect); virtual;
      Procedure   CooHandler (bBoundsOld, bBounds : BRect); virtual;
      Procedure   DeltaMirrorX;
      Procedure   DeltaMirrorY;
      Procedure   SetBounds (bx1, by1, bx2, by2 : integer); virtual;
      Procedure   SetDelta (bdlt : BDelta);
      Procedure   SetAccel (bacc : integer);
      Procedure   SetAngle (bang : integer);
      Procedure   IncAngle (dang : integer);
      Procedure   SetSpeed (bspd : integer);
      Procedure   GetDelta (var bdlt : BDelta);
      Function    GetAccel : integer;
      Function    GetSpeed : integer;
  end;

  BListPtr = ^BList;
  BList    = record
    ObjectP : BObjectPtr;
    Next    : BListPtr
  end;

  BObjectListPtr = ^BObjectList;
  BObjectList    = object
    private
      CurPtr        : BListPtr;
      List          : BListPtr;
    public
      Constructor Init;
      Procedure   AddObject (BObjectP : BObjectPtr);
      Procedure   RemoveObject (var Ptr : BListPtr);
      Procedure   DrawAll;
      Procedure   LiveAll;
      Function    GetCurObject : BObjectPtr;
      Procedure   GetFirstObject (var BList    : BListPtr;
                                  var BObjectP : BObjectPtr);
      Procedure   GetNextObject  (var BList    : BListPtr;
                                  var BObjectP : BObjectPtr);
      Destructor  Done;
      Function    IsEmptyList : boolean;
  end;

  function  BrkX (i : integer) : integer;
  function  BrkY (i : integer) : integer;
  procedure GetBrickIndex (bx, by : integer; var iy, ix : integer);

implementation

function BrkX (i : integer) : integer;
begin
  BrkX := L_MARGIN + i * BRK_WDT
end;

function BrkY (i : integer) : integer;
begin
  BrkY := T_MARGIN + i * BRK_HGT
end;

procedure GetBrickIndex (bx, by : integer; var iy, ix : integer);
begin
  ix := (bx - L_MARGIN) div BRK_WDT;
  iy := (by - T_MARGIN) div BRK_HGT
end;

constructor BPrizeSet.Init;
begin
  PrizeSet := [];
  ClearSet
end;

procedure BPrizeSet.AddPrize (PrizeType : BPrizeType);
begin
  PrizeSet := PrizeSet + [PrizeType];
  AddedSet := AddedSet + [PrizeType]
end;

procedure BPrizeSet.RemovePrize (PrizeType : BPrizeType);
begin
  PrizeSet   := PrizeSet - [PrizeType];
  RemovedSet := RemovedSet + [PrizeType]
end;

procedure BPrizeSet.ClearSet;
begin
  AddedSet   := [];
  RemovedSet := []
end;

function BPrizeSet.InPrizeSet (PrizeType : BPrizeType) : boolean;
begin
  InPrizeSet := PrizeType in PrizeSet
end;

function BPrizeSet.InAddedSet (PrizeType : BPrizeType) : boolean;
begin
  InAddedSet := PrizeType in AddedSet
end;

function BPrizeSet.InRemovedSet (PrizeType : BPrizeType) : boolean;
begin
  InRemovedSet := PrizeType in RemovedSet
end;

constructor BField.Init;
begin
  Assign (F, FIELDS_FILE);
  Reset (F)
end;

destructor BField.Done;
begin
  Close (F)
end;

function BField.EOFields : boolean;
begin
  EOFields := EOF (F)
end;

procedure BField.SeekField (nf : integer);
begin
  Seek (F, nf)
end;

procedure BField.UnpackField (var PackedField : BPackedField);
var
  i, j : integer;
begin
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do begin
      Field [i, j].Color    := PackedField.Field [i, j] and $07;
      Field [i, j].BrkType  := BBrickType ((PackedField.Field [i, j] and $F8) shr 3);
    end;
  Name := PackedField.Name
end;

function BField.GetNumOfBricks : integer;
var
  n    : integer;
  i, j : integer;
begin
  n := 0;
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do
      with Field [i, j] do
        case BrkType of
          T_ONCE, T_TWICE, T_THREE, T_GOLDED, T_EXPLODED, T_MOVED : Inc (n);
        end;
  GetNumOfBricks := n
end;

procedure BField.LoadNextField;
var
  PackedField : BPackedField;
begin
  Read (f, PackedField);
  UnpackField (PackedField);
  SetBricksCount (GetNumOfBricks)
end;

procedure BField.GetBrick (i, j : integer; var Brk : BBrick);
begin
  if (i in [0..FIELD_HGT]) and (j in [0..FIELD_WDT]) then
    Brk := Field [i, j]
  else
    Brk.BrkType := T_EMPTY
end;

procedure BField.SetBrick (i, j : integer; Brk : BBrick);
begin
  Field [i, j] := Brk
end;

procedure BField.DrawBrick (i, j : integer);
begin
end;

procedure BField.DrawField;
var
  i, j : integer;
begin
  for i := 0 to FIELD_HGT do
    for j := 0 to FIELD_WDT do
      DrawBrick (i, j)
end;

procedure BField.IncBricksCount (dc : integer);
begin
  Inc (BricksCount, dc)
end;

procedure BField.CollisionHandler (dlt : BDelta; i, j : integer;
                                   exploded, moved : boolean);
begin
end;

function BField.DetectCollision (dlt : BDelta; bx1, by1, bx2, by2 : integer;
                                 Exploded : boolean) : boolean;
var
  Res : boolean;

  procedure Check (bx, by : integer);
  var
    i, j : integer;
    Brk  : BBrick;
  begin
    if not Res then begin
      GetBrickIndex (bx, by, i, j);
      if (Field [i, j].BrkType <= T_MOVED) and
         (Field [i, j].BrkType <> T_EMPTY) and
         (i in [0..FIELD_HGT]) and (j in [0..FIELD_WDT]) then begin
        Res := True;
        CollisionHandler (dlt, i, j, Exploded, False);
        if Exploded then begin
          CollisionHandler (dlt, i-1, j-1, Exploded, False);
          CollisionHandler (dlt, i-1, j, Exploded, False);
          CollisionHandler (dlt, i-1, j+1, Exploded, False);
          CollisionHandler (dlt, i, j+1, Exploded, False);
          CollisionHandler (dlt, i+1, j+1, Exploded, False);
          CollisionHandler (dlt, i+1, j, Exploded, False);
          CollisionHandler (dlt, i+1, j-1, Exploded, False);
          CollisionHandler (dlt, i, j-1, Exploded, False)
        end
      end
    end
end;

begin
  Res := False;
  Check (bx1, by1);
  Check (bx2, by1);
  Check (bx1, by2);
  Check (bx2, by2);
  DetectCollision := Res
end;

procedure BField.SetBricksCount (BrCount : integer);
begin
  BricksCount := BrCount
end;

function BField.GetBricksCount;
begin
  GetBricksCount := BricksCount
end;

function BField.GetFieldName : string;
begin
  GetFieldName := Name
end;

constructor BObject.Init (bx1, by1, bx2, by2 : integer);
begin
  SetBounds (bx1, by1, bx2, by2);
  BoundsOld := Bounds;
  ToDraw (bx1, by1);
  RefreshFlag := False;
  DieFlag     := False
end;

procedure BObject.ToLive (var bBounds : BRect);
begin
end;

procedure BObject.ToDraw (x, y : integer);
begin
end;

procedure BObject.ToDie;
begin
  DieFlag := True
end;

procedure BObject.SetBounds (bx1, by1, bx2, by2 : integer);
begin
  with Bounds do begin
    x1 := bx1;
    y1 := by1;
    x2 := bx2;
    y2 := by2
  end
end;

procedure BObject.GetBounds (var bBounds : BRect);
begin
  bBounds := Bounds
end;

procedure BObject.Refresh;
begin
  RefreshFlag := True
end;

function BObject.GetX1 : integer;
begin
  GetX1 := Bounds.x1
end;

function BObject.GetY1 : integer;
begin
  GetY1 := Bounds.y1
end;

function BObject.GetX2 : integer;
begin
  GetX2 := Bounds.x2
end;

function BObject.GetY2 : integer;
begin
  GetY2 := Bounds.y2
end;

destructor BObject.Done;
begin
  with BoundsOld do
    ToDraw (x1, y1)
end;

constructor BAnimation.Init (i, j, bFrames, bSpeed : integer);
begin
  NFrame := 0;
  Time   := 0;
  Frames := bFrames;
  Speed  := bSpeed;
  Inherited Init (BrkX (j), BrkY (i), 0, 0);
  DrawFrame (GetX1, GetY1, 0)
end;

procedure BAnimation.ToLive (var bBounds : BRect);
begin
  Inc (Time);
  if Time >= Speed then begin
    Inc (NFrame);
    if NFrame >= Frames then
      ToDie;
    DrawFrame (bBounds.x1, bBounds.y1, NFrame);
    Time := 0
  end
end;

procedure BAnimation.DrawFrame (x, y, n : integer);
begin
end;

destructor BAnimation.Done;
var
  i : integer;
begin
  for i := NFrame+1 to Frames do
    DrawFrame (GetX1, GetY1, i)
end;

constructor BGravityObject.Init (bx1, by1, bx2, by2 : integer;
				 bdlt : BDelta);
begin
  Inherited Init (bx1, by1, bx2, by2);
  SetDelta (bdlt);
  with RealBounds do begin
    rx1 := bx1; ry1 := by1;
    rx2 := bx2; ry2 := by2
  end
end;

procedure BGravityObject.ToLive (var bBounds : BRect);
var
  bNewBounds  : BRect;
  rx, ry      : BReal;
  TRealBounds : BRealRect;
begin
  with TRealBounds do
    with Delta do begin
      rx  := dx * speed;
      ry  := dy * speed;
      rx1 := RealBounds.rx1 + rx; rx2 := RealBounds.rx2 + rx;
      ry1 := RealBounds.ry1 + ry; ry2 := RealBounds.ry2 + ry;
      dy  := dy + accel*speed
    end;
  with TRealBounds do
    with bNewBounds do begin
      x1 := round (rx1); x2 := round (rx2);
      y1 := round (ry1); y2 := round (ry2)
    end;
  DeltaChange := False;
  CooHandler (bBounds, bNewBounds);
  if not DeltaChange then begin
    bBounds    := bNewBounds;
    RealBounds := TRealBounds
  end
end;

procedure BGravityObject.CooHandler (bBoundsOld, bBounds : BRect);
begin
  with bBounds do begin
    if (x2 > R_MARGIN) or (x1 < L_MARGIN) then
      DeltaMirrorX;
    if (y1 < T_MARGIN) then
      DeltaMirrorY;
    if y2 > B_MARGIN then
      ToDie
  end
end;

procedure BGravityObject.DeltaMirrorX;
begin
  with Delta do
    dx := -dx;
  DeltaChange := True
end;

procedure BGravityObject.DeltaMirrorY;
begin
  with Delta do
    dy := -dy;
  DeltaChange := True
end;

procedure BGravityObject.SetBounds (bx1, by1, bx2, by2 : integer);
begin
  DeltaChange := True;
  Inherited SetBounds (bx1, by1, bx2, by2);
  with RealBounds do begin
    rx1 := bx1;
    rx2 := bx2;
    ry1 := by1;
    ry2 := by2
  end
end;

procedure BGravityObject.SetDelta (bdlt : BDelta);
begin
  Delta       := bdlt;
  DeltaChange := True
end;

procedure BGravityObject.GetDelta (var bdlt : BDelta);
begin
  bdlt := Delta
end;

procedure BGravityObject.SetSpeed (bspd : integer);
begin
  Delta.Speed := bspd / KSpeed
end;

function BGravityObject.GetSpeed : integer;
begin
  GetSpeed := round (Delta.Speed * KSpeed)
end;

procedure BGravityObject.IncAngle (dang : integer);
var
  rad        : BReal;
  cosr, sinr : BReal;
  dxn, dyn   : BReal;
begin
  rad  := pi/180 * dang;
  cosr := cos (rad);
  sinr := sin (rad);
  DeltaChange := True;
  with Delta do begin
    dxn := dx * cosr - dy * sinr;
    dyn := dx * sinr + dy * cosr;
    dx  := dxn;
    dy  := dyn;
  end
end;

procedure BGravityObject.SetAngle (bang : integer);
var
  rad : BReal;
begin
  rad := pi/180 * bang;
  DeltaChange := True;
  with Delta do begin
    dx := cos (rad);
    dy := sin (rad)
  end
end;

procedure BGravityObject.SetAccel (bacc : integer);
begin
  Delta.Accel := bacc/KAccel
end;

function BGravityObject.GetAccel : integer;
begin
  GetAccel := round (Delta.Accel*KAccel)
end;

constructor BObjectList.Init;
begin
  New (List);
  with List^ do begin
    Next    := NIL;
    ObjectP := NIL
  end
end;

procedure BObjectList.AddObject (BObjectP : BObjectPtr);
var
  Ptr : BListPtr;
begin
  New (Ptr);
  with Ptr^ do begin
    ObjectP := BObjectP;
    Next    := List
  end;
  List := Ptr
end;

procedure BObjectList.RemoveObject (var Ptr : BListPtr);
var
  NPtr : BListPtr;
begin
  NPtr := Ptr^.Next;
  Dispose (Ptr^.ObjectP, Done);
  Ptr^.Next    := NPtr^.Next;
  Ptr^.ObjectP := NPtr^.ObjectP;
  Dispose (NPtr);
end;

procedure BObjectList.DrawAll;
begin
  CurPtr := List;
  while CurPtr^.Next <> NIL do with CurPtr^.ObjectP^ do begin
    if (Bounds.x1 <> BoundsOld.x1) or
       (Bounds.y1 <> BoundsOld.y1) or RefreshFlag then begin
          RefreshFlag := False;
          with BoundsOld do
	    ToDraw (x1, y1);
          with Bounds do
	    ToDraw (x1, y1);
          BoundsOld := Bounds
    end;
    CurPtr := CurPtr^.Next
  end
end;

procedure BObjectList.LiveAll;
begin
  CurPtr := List;
  while CurPtr^.Next <> NIL do with CurPtr^.ObjectP^ do begin
    ToLive (Bounds);
    if DieFlag then
      RemoveObject (CurPtr)
    else
      CurPtr := CurPtr^.Next
  end
end;

function BObjectList.GetCurObject : BObjectPtr;
begin
  GetCurObject := CurPtr^.ObjectP
end;

procedure BObjectList.GetFirstObject (var BList : BListPtr;
                                      var BObjectP : BObjectPtr);
begin
  BList    := List;
  BObjectP := BList^.ObjectP;
end;

procedure BObjectList.GetNextObject (var BList : BListPtr;
                                     var BObjectP : BObjectPtr);
begin
  with BList^ do begin
    BObjectP := Next^.ObjectP;
    BList    := Next
  end
end;

function BObjectList.IsEmptyList : boolean;
begin
  IsEmptyList := List^.Next = NIL
end;

destructor BObjectList.Done;
var
  NPtr : BListPtr;
begin
  while List <> NIL do begin
    if List^.ObjectP <> NIL then
      Dispose (List^.ObjectP, Done);
    NPtr := List^.Next;
    Dispose (List);
    List := NPtr
  end
end;

begin
end.
