unit Shell;

interface

uses Crt, Dos, Graph, Imager;

type
  TKeyType = (LEFT, RIGHT, UP, DOWN, FIRE);

  TKeys = array [LEFT..FIRE] of byte;

const
  cmEscape  = -1;
  cmFire    = -2;
  cmOk      = -3;
  cmCancel  = -4;
  cmNone    = -5;

  BTN_HEIGHT = 1;

  Escape     = 1;
  F1         = 59;
  BackSpace  = 14;
  Tab        = 15;
  Enter      = 28;
  Ctrl       = 29;
  LeftShift  = 42;
  RightShift = 54;
  Alt        = 56;
  Space      = 57;
  LeftArrow  = 75;
  RightArrow = 77;
  UpArrow    = 72;
  DownArrow  = 80;
  Home       = 71;
  EndK       = 79;
  PageUp     = 73;
  PageDown   = 81;
  InsertK    = 82;
  DeleteK    = 83;
  CapsLock   = 58;
  NumLock    = 69;
  Minus      = 74;
  Plus       = 78;

  StandardKeys : TKeys =
    (LeftArrow, RightArrow, UpArrow, DownArrow, Enter);

  Keys1        : TKeys =
    (16, 17, 255, 255, Ctrl);
  Keys2        : TKeys =
    (24, 25, 255, 255, Space);

  StandardCursor : array [1..32] of word =
    ($BFFF, $1FFF, $0FFF, $07FF, $03FF, $01FF, $00FF, $007F,
     $003F, $001F, $003F, $03FF, $0FFF, $0FFF, $1FFF, $BFFF,
     $0000, $4000, $6000, $7000, $7800, $7C00, $7E00, $7F00,
     $7F80, $7FC0, $7C00, $7000, $6000, $6000, $4000, $0000);

  AsciiLo : string [57] =
    (#0+'1234567890-='+#0#0+'qwertyuiop[]'+#0#0+
     'asdfghjkl;'''+#0#0+'\zxcvbnm,./'+#0+'*'+#0+' ');
  AsciiHi : string [57] =
    (#0+'!@#$%^&*()_+'+#0#0+'QWERTYUIOP{}'+#0#0+
     'ASDFGHJKL:"'+#0#0+'|ZXCVBNM<>?'+#0+'*'+#0+' ');
  AsciiRusLo : string [57] =
    (#0+'1234567890-='+#0#0+'㪥'+#0#0+
     '뢠஫'+#0#0+'\ᬨ/'+#0+'*'+#0+' ');
  AsciiRusHi : string [57] =
    (#0+'!"/#:''.;?%_+'+#0#0+''+#0#0+
     ''+#0#0+'|?'+#0+'*'+#0+' ');
var
  CursorState  : boolean;
  MousePresent : boolean;

procedure HideCursor;
procedure ShowCursor;
procedure SetMousePage (Page : byte);
procedure SetCursorStyle (var ScrCurMask; X, Y : byte);
procedure KillArea;
procedure InitArea (MaxHorizRegion : byte);
procedure AddButton (Command : integer; x, y, wdt, hgt : word);
procedure AddSprButton (Command : integer; x, y : word;
			GrLib : TGrLib; Name : string);
procedure AddTextButton (Command : integer; x, y, len, hgt : word;
			 Text : string);
procedure SetMousePort (ix1, iy1, ix2, iy2 : integer);
procedure SetStepToPixel (step: word);
procedure MouseGotoXY (XC, YC : integer);
procedure SetMouseAspect (Aspect : real);
procedure ShellInit;
procedure ShellDone;
procedure WaitEvent (Keys : TKeys);
procedure ClearMouseDelta;
procedure ClearShellEvent;
procedure ClearShellKeys;
function  GetKeyString (key : byte) : string;
function  KeyPressed  (Key : byte) : boolean;
function  KeyCurrent  (Key : byte) : boolean;
function  KeyReleased (Key : byte) : boolean;
function  WhatPressed    : byte;
function  ReadKey        : char;
function  AnyKey         : boolean;
function  GetShellEvent  : integer;
function  GetMouseDeltaX : integer;
function  GetMouseDeltaY : integer;
function  GetMouseAspect : real;

implementation

type
  TRegion = record
    x1, y1 : word;
    x2, y2 : word;
    Cmd    : integer;
    Next   : pointer;
    Prev   : pointer
  end;
  TRegionPtr = ^TRegion;

var
  MouseAspect   : real;

  MouseDeltaX   : integer;
  MouseDeltaY   : integer;
  OldX          : integer;
  OldY          : integer;

  ShellEvent    : integer;

  FirePressed   : boolean;
  OldCheck      : boolean;

  BtnImage      : TImagePtr;

  HeadPtr       : TRegionPtr;
  NewPtr        : TRegionPtr;
  TailPtr       : TRegionPtr;
  XPtr          : TRegionPtr;
  OldRegion     : TRegionPtr;
  HMaxRegion    : byte;

  Reg           : registers;
  OldInt9       : pointer;

  CurVis        : boolean;
  CapsLockFlag  : boolean;
  RusFlag       : boolean;

  PressedKeys   : set of byte;
  CurrentKeys   : set of byte;
  ReleasedKeys  : set of byte;

procedure CursorVisible (Vis : boolean);
begin
  if CurVis <> Vis and MousePresent then
    with Reg do begin
      if Vis then
        ax := $01
      else
        ax := $02;
      Intr ($33, Reg)
    end;
  CurVis := Vis
end;

procedure ResetMouse;
begin
  with Reg do begin
    ax := 0;
    Intr ($33, Reg);
    MousePresent := ax=$FFFF
  end
end;

procedure SetMousePage (Page : byte);
begin
  with Reg do begin
    ax := $1D;
    bx := Page;
    Intr ($33, Reg)
  end
end;

procedure SetCursorStyle (var ScrCurMask; X, Y : byte);
begin
  if MousePresent then
    with Reg do begin
      ax := $9;
      bx := X;
      cx := Y;
      es := Seg (ScrCurMask);
      dx := Ofs (ScrCurMask);
      Intr ($33, Reg)
    end
end;

procedure Button (Region : TRegionPtr; Dir : Dir3D);
var
  DH, i : byte;
begin
  CursorVisible (False);
  DH := ord (Dir) * BTN_HEIGHT;
  with Region^ do begin
    if (OldRegion <> Region) and (Dir = DOWN_DIR) then begin
      FreeImage (BtnImage);
      CreateImage (BtnImage, x2 - x1, y2 - y1);
      GetImage (x1, y1, BtnImage)
    end;
    if Dir = UP_DIR then
      Frame3D (x1, y1, x2, y2, BTN_HEIGHT, Dir)
    else begin
      SetColor (BottomColor);
      for i := 1 to DH do begin
        Line (x1-i, y1-i, x1-i, y2);
	Line (x1-i, y1-i, x2, y1-i)
      end
    end;
    PutImage (x1 + DH, y1 + DH, BtnImage, NormalPut)
  end;
  CursorVisible (CursorState)
end;

procedure XPointer (Region : TRegionPtr; Visible : boolean);
var
  d : integer;
begin
  d := BTN_HEIGHT+1;
  CursorVisible (False);
  if Visible then
    SetColor (Black)
  else
    SetColor (MidColor);
  with Region^ do
    Rectangle (x1 - d, y1 - d, x2 + d, y2 + d);
  SetLineStyle (UserBitLn, $5555, NormWidth);
  with Region^ do
    Rectangle (x1 + 1, y1 + 1, x2 - 1, y2 - 1);
  SetLineStyle (SolidLn, 0, NormWidth);
  CursorVisible (CursorState)
end;

procedure HideCursor;
begin
  CursorVisible (False);
  CursorState := False
end;

procedure ShowCursor;
begin
  CursorState := True;
  CursorVisible (True);
  XPointer (XPtr, True)
end;

procedure KillArea;
var
  Ptr : TRegionPtr;
begin
  repeat
    Ptr := HeadPtr^.Next;
    FreeMem (HeadPtr, sizeof (TRegion));
    HeadPtr := Ptr
  until HeadPtr = NIL;
  FreeImage (BtnImage);
  OldRegion := NIL;
  XPtr      := NIL;
  TailPtr   := NIL
end;

procedure InitArea (MaxHorizRegion : byte);
begin
  if HeadPtr <> NIL then
    KillArea;
  HideCursor;
  HeadPtr    := NIL;
  XPtr       := NIL;
  TailPtr    := NIL;
  OldRegion  := NIL;
  HMaxRegion := MaxHorizRegion
end;

procedure AddButton (Command : integer; x, y, wdt, hgt : word);
var
  Ptr : TRegionPtr;
  d   : byte;
begin
  d := BTN_HEIGHT + 1;
  GetMem (Ptr, sizeof (TRegion));
  with Ptr^ do begin
    x1    := x - 3;
    y1    := y - 3;
    x2    := x + Wdt + 3;
    y2    := y + Hgt + 3;
    Cmd   := Command;
    Next  := HeadPtr;
    Frame3D (x1, y1, x2, y2, BTN_HEIGHT, UP_DIR);
    SetColor (15);
    if HeadPtr <> NIL then
      HeadPtr^.Prev := Ptr
    else begin
      TailPtr := Ptr;
      XPtr := Ptr
    end;
    Prev := NIL;
    HeadPtr := Ptr
  end
end;

procedure AddSprButton (Command : integer; x, y : word;
                        GrLib : TGrLib; Name : string);
var
  Image : TImagePtr;
begin
  LoadSprite (GrLib, Image, Name);
  PutImage (x, y, Image, NormalPut);
  AddButton (Command, x, y, GetImageWdt (Image), GetImageHgt (Image));
  FreeImage (Image)
end;

procedure AddTextButton (Command : integer; x, y, len, hgt : word;
                         Text : string);
var
  OldColor : byte;
begin
  OldColor := GetColor;
  SetTextJustify (CenterText, CenterText);
  OutTextXY (x + len div 2, y + hgt div 2 - 1, Text);
  SetTextJustify (LeftText, TopText);
  AddButton (Command, x, y, len, hgt);
  SetColor (OldColor)
end;

procedure SetMousePort (ix1, iy1, ix2, iy2 : integer);
begin
  if (ix1 < 0) or (iy1 < 0) or
     (ix2 > GetMaxX) or (iy2 > GetMaxY) then
        Error ('Wrong mouse port coordinates !');
  if MousePresent then
    with Reg do begin
      ax := $7;
      cx := ix1;
      dx := ix2;
      Intr ($33, Reg);
      ax := $8;
      cx := iy1;
      dx := iy2;
      Intr ($33, Reg)
    end
end;

procedure SetStepToPixel (step: word);
begin
  if MousePresent then
    with Reg do begin
      ax := $0F;
      cx := step;
      dx := step;
      Intr ($33, Reg)
    end
end;

procedure MouseGotoXY (XC, YC : integer);
begin
  if MousePresent then
    with Reg do begin
      ax := $04;
      cx := XC;
      dx := YC;
      Intr ($33, Reg)
    end
end;

function InRegion (XC, YC : integer; Region : TRegionPtr) : boolean;
begin
  with Region^ do
    InRegion := (XC >= x1) and (XC <= x2) and
                (YC >= y1) and (YC <= y2)
end;

function GetRegionPtr (XC, YC : integer) : TRegionPtr;
var
  Ptr : TRegionPtr;
begin
  Ptr := HeadPtr;
  repeat
    if InRegion (XC, YC, Ptr) then begin
      GetRegionPtr := Ptr;
      Exit
    end;
    Ptr := Ptr^.Next
  until Ptr = NIL;
  GetRegionPtr := Ptr
end;

procedure ClickHandler (XC, YC, Mask : integer);
begin
  if (Mask and $02) = $02 then begin
    NewPtr   := GetRegionPtr (XC, YC);
    OldCheck := True
  end;
  if (Mask and $04) = $04 then begin
    if (NewPtr <> NIL) and not OldCheck then begin
      FirePressed := False;
      Button (NewPtr, UP_DIR);
      ShellEvent := NewPtr^.Cmd
    end;
    NewPtr := NIL
  end;
  if NewPtr <> NIL then
    if InRegion (XC, YC, NewPtr) then begin
      if OldCheck and not FirePressed then begin
        XPointer (XPtr, False);
        XPtr := NewPtr;
        XPointer (XPtr, True);
	Button (NewPtr, DOWN_DIR);
        FirePressed := True;
        OldCheck    := False
      end
    end else
      if not OldCheck then begin
	XPointer (XPtr, False);
	XPtr := NewPtr;
	XPointer (XPtr, True);
	Button (NewPtr, UP_DIR);
	FirePressed := False;
	OldCheck    := True
      end
end;

procedure MouseHandler (Mask, Buttons, XC, YC, SDY, SDX : integer); far;
begin
  Inc (MouseDeltaX, SDX - OldX);
  Inc (MouseDeltaY, SDY - OldY);
  OldX := SDX;
  OldY := SDY;
  if CursorState and (HeadPtr <> NIL) then
    ClickHandler (XC, YC, Mask)
  else
    if (Mask and $04) = $04 then
      ShellEvent := cmFire
end;

procedure MouseHandlerProc; far; assembler;
asm
  push bp
  push ds
  push es
  push ax
  mov ax, SEG @DATA
  mov ds,ax
  push bx
  push cx
  push dx
  push di
  push si
  call [MouseHandler]
  pop es
  pop ds
  pop bp
  ret far
end;

procedure SetMouseHandler;
begin
  if MousePresent then
    with Reg do begin
      ax := $0C;
      cx := 7;
      es := seg (MouseHandlerProc);
      dx := ofs (MouseHandlerProc);
      Intr ($33, Reg)
    end
end;

procedure SetMouseAspect (Aspect : real);
begin
  MouseAspect := Aspect
end;

function GetMouseAspect : real;
begin
  GetMouseAspect := MouseAspect
end;

function GetMouseDeltaX : integer;
begin
  GetMouseDeltaX := round (MouseDeltaX * MouseAspect)
end;

function GetMouseDeltaY : integer;
begin
  GetMouseDeltaY := round (MouseDeltaY * MouseAspect)
end;

procedure ClearMouseDelta;
begin
  MouseDeltaX := 0;
  MouseDeltaY := 0
end;

procedure ClearMouseHandler;
begin
  if MousePresent then begin
    with Reg do begin
      ax := $0C;
      cx := 0;
      es := 0;
      dx := 0;
      Intr ($33, Reg)
    end
  end
end;

procedure KeyboardHandler; interrupt;
var
  ScanCode : byte;
  Bit7     : boolean;
begin
  ClearShellKeys;
  ScanCode     := Port [$60];
  Bit7         := (ScanCode and $80) <> 0;
  ScanCode     := ScanCode and $7F;
  if Bit7 then begin
    if KeyCurrent (ScanCode) then begin
      CurrentKeys  := CurrentKeys  - [ScanCode];
      ReleasedKeys := ReleasedKeys + [ScanCode]
  end end else
    if not KeyCurrent (ScanCode) then begin
      CurrentKeys := CurrentKeys + [ScanCode];
      PressedKeys := PressedKeys + [ScanCode]
    end;
  asm
    mov al, $20
    out $20, al
  end
end;

procedure SetKeyboardHandler;
begin
  CurrentKeys  := [];
  ClearShellKeys;
  CapsLockFlag := False;
  RusFlag      := False;
  GetIntVec ($9, OldInt9);
  SetIntVec ($9, @KeyboardHandler)
end;

procedure ClearKeyboardHandler;
begin
  SetIntVec ($9, OldInt9)
end;

procedure ShellInit;
begin
  ResetMouse;
  HideCursor;
  SetStepToPixel (10);
  SetMouseAspect (1);
  MouseGotoXY (GetMaxX div 2, GetMaxY div 2);
  MouseDeltaX := 0;
  MouseDeltaY := 0;
  OldX        := 0;
  OldY        := 0;
  SetCursorStyle (StandardCursor, 1, 1);
  InitArea (1);
  SetMouseHandler;
  SetKeyboardHandler
end;

procedure ShellDone;
begin
  if HeadPtr <> NIL then
    KillArea;
  ClearMouseHandler;
  ClearKeyboardHandler
end;

procedure ClearShellKeys;
begin
  ForbidInt;
  PressedKeys  := [];
  ReleasedKeys := [];
  AllowInt
end;

function KeyCurrent (Key : byte) : boolean;
begin
  ForbidInt;
  KeyCurrent := Key in CurrentKeys;
  AllowInt
end;

function KeyPressed (Key : byte) : boolean;
begin
  ForbidInt;
  KeyPressed  := Key in PressedKeys;
  PressedKeys := PressedKeys - [Key];
  AllowInt
end;

function KeyReleased (Key : byte) : boolean;
begin
  ForbidInt;
  KeyReleased  := Key in ReleasedKeys;
  ReleasedKeys := ReleasedKeys - [Key];
  AllowInt
end;

function AnyKey : boolean;
begin
  ForbidInt;
  AnyKey      := PressedKeys <> [];
  PressedKeys := [];
  AllowInt;
end;

function WhatPressed : byte;
var
  i : integer;
begin
  WhatPressed := 0;
  ForbidInt;
  for i := 0 to 255 do
    if i in CurrentKeys then
      WhatPressed := i;
  AllowInt
end;

function ReadKey : char;
var
  i : integer;
begin
  ReadKey := #0;
  if KeyCurrent (CapsLock) then begin
    CapsLockFlag := not CapsLockFlag;
    Beep (440+40*ord (CapsLockFlag))
  end;
  if KeyCurrent (Ctrl) then begin
    RusFlag := not RusFlag;
    Beep (300+40*ord (RusFlag))
  end;
  for i := 1 to 57 do
    if (i in CurrentKeys) and (i <> LeftShift) and (i <> RightShift) then
         if KeyCurrent (LeftShift) or KeyCurrent (RightShift) or CapsLockFlag then
           if RusFlag then
             ReadKey := AsciiRusHi [i]
           else
             ReadKey := AsciiHi [i]
         else
           if RusFlag then
             ReadKey := AsciiRusLo [i]
           else
             ReadKey := AsciiLo [i]
end;

procedure MovePointer (dx, dy : integer);
var
  Steps     : integer;
  Direction : boolean;
begin
  if not FirePressed and (HeadPtr <> NIL) and CursorState then
    if XPtr <> NIL then begin
      Steps     := dy * HMaxRegion + dx;
      Direction := Steps >= 0;
      Steps     := Abs (Steps);
      XPointer (XPtr, False);
      repeat
	if Direction then
	  XPtr := XPtr^.Prev
	else
	  XPtr := XPtr^.Next;
	if XPtr = NIL then
	  if Direction then
	    XPtr := TailPtr
	  else
	    XPtr := HeadPtr;
	Dec (Steps)
      until (Steps = 0);
      XPointer (XPtr, True)
    end else begin
      XPtr := TailPtr;
      XPointer (XPtr, True)
    end
end;

procedure FireStarter;
begin
  if (HeadPtr = NIL) or not CursorState then
    ShellEvent := cmFire
  else
    if XPtr <> NIL then
      if not FirePressed then begin
	FirePressed := True;
	Button (XPtr, DOWN_DIR)
      end;
end;

procedure FireExtinguisher;
begin
  if FirePressed then begin
    ShellEvent := XPtr^.Cmd;
    Button (XPtr, UP_DIR);
    FirePressed := False
  end
end;

procedure WaitEvent (Keys : TKeys);
begin
  ClearShellEvent;
  repeat
    if KeyPressed  (Keys [FIRE])  then FireStarter;
    if KeyReleased (Keys [FIRE])  then FireExtinguisher;
    if KeyPressed  (Keys [LEFT])  then MovePointer (-1, 0);
    if KeyPressed  (Keys [RIGHT]) then MovePointer (1,  0);
    if KeyPressed  (Keys [UP])    then MovePointer (0, -1);
    if KeyPressed  (Keys [DOWN])  then MovePointer (0,  1);
    if KeyPressed  (Escape)       then ShellEvent := cmEscape
  until ShellEvent <> cmNone
end;

function GetKeyString (key : byte) : string;
var
  s : string;
begin
  s := '';
  if (Key in [1..57]) and (AsciiLo [key] <> #0) then
    s := UpCase (AsciiLo [key]);
  case key of
    BackSpace  : s := 'BSpace';
    Tab        : s := 'Tab';
    Enter      : s := 'Enter';
    Space      : s := 'Space';
    Home       : s := 'Home';
    UpArrow    : s := chr (24);
    PageUp     : s := 'PgUp';
    LeftArrow  : s := chr (27);
    RightArrow : s := chr (26);
    EndK       : s := 'End';
    DownArrow  : s := chr (25);
    PageDown   : s := 'PgDown';
    InsertK    : s := 'Insert';
    DeleteK    : s := 'Delete';
    Ctrl       : s := 'Ctrl';
    LeftShift  : s := 'LShift';
    RightShift : s := 'RShift';
    Alt        : s := 'Alt';
    CapsLock   : s := 'CapsLock';
    NumLock    : s := 'NumLock';
    Minus      : s := '-';
    Plus       : s := '+'
  end;
  GetKeyString := s
end;

procedure ClearShellEvent;
begin
  ShellEvent := cmNone
end;

function GetShellEvent : integer;
begin
  GetShellEvent := ShellEvent
end;

end.
